@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2018, 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2018, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2018, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2018, 2020, 2022, 2023 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2018, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2020 Johri van Eerd <vaneerd.johri@gmail.com>
@@ -45,6 +45,7 @@
             add-function-return
             add-reply-port
             binding-into-blocking
+            extract-call
             not-or-guards
             normalize:event
             normalize:event+illegals
@@ -247,9 +248,9 @@
           (match guards
             (()
              (make <literal> #:value "true"))
-	    ((guard)
+            ((guard)
              (make <not> #:expression (.expression guard)))
-	    ((h t ...)
+            ((h t ...)
              (let ((expressions (map (compose (cute make <not> #:expression <>)
                                               .expression)
                                      guards)))
@@ -353,22 +354,6 @@
                   triples)))
     (make <declarative-compound> #:elements st)))
 
-(define (declarative-illegal? o)
-  (match o
-    (($ <illegal>) o)
-    (($ <declarative-illegal>) o)
-    ((and ($ <compound>) (= ast:statement* (? null?))) #f)
-    (($ <compound>) (declarative-illegal? ((compose car ast:statement*) o)))
-    (_ #f)))
-
-(define ((triples:fix-empty-interface model) triples)
-  (if (and (is-a? model <interface>) (null? triples))
-      (let* ((on (make <on> #:triggers (make <triggers> #:elements (list (make <trigger> #:event.name "inevitable")))))
-             (guard (make <guard> #:expression (make <literal> #:value "false")))
-             (statement (make <compound> #:elements (list (make <illegal>)))))
-        (list (make-triple on guard #f statement)))
-      triples))
-
 (define (trigger-equal? a b)
   (and (equal? (.port.name a) (.port.name b))
        (equal? (.event.name a) (.event.name b))))
@@ -398,31 +383,42 @@
 (define (triples:mark-the-end triples)
   (define (mark-the-end t)
     (let* ((on (triple-on t))
-           (illegal? (declarative-illegal? (triple-statement t)))
+           (illegal? (ast:illegal? (triple-statement t)))
            (blocking? (triple-blocking? t))
-           (valued-trigger? (ast:typed? ((compose car ast:trigger*) on)))
-           (modeling? (is-a? ((compose .event car ast:trigger*) on) <modeling-event>))
-           (port ((compose .port car ast:trigger*) on))
-           (provides? (and port (ast:provides? port))))
-      (if (ast:parent on <interface>)
-          (if (or valued-trigger? illegal?) t
-              (let* ((statement (triple-statement t))
-                     (end (make (if modeling? <the-end> <reply>)))
-                     (elements (if (is-a? statement <compound>)
-                                   (append (ast:statement* statement) (list end))
-                                   (list statement end)))
-                     (statement (make <compound> #:elements elements)))
-                (make-triple on (triple-guard t) (triple-blocking? t) statement)))
-          (let ((t (if (or valued-trigger? illegal? blocking?) t
-                       (let* ((statement (triple-statement t))
-                              (reply (if provides? (list (make <reply>)) '()))
-                              (elements (if (is-a? statement <compound>)
-                                            (ast:statement* statement)
-                                            (list statement)))
-                              (elements (append elements reply))
-                              (statement (make <compound> #:elements elements)))
-                         (make-triple on (triple-guard t) (triple-blocking? t) statement)))))
-            (if illegal? t (add-the-end t))))))
+           (typed-trigger? (ast:typed? ((compose car ast:trigger*) on))))
+      (cond
+       (illegal?
+        t)
+       ((or typed-trigger? blocking?)
+        (add-the-end t))
+       (else
+        (let* ((event ((compose .event car ast:trigger*) on))
+               (modeling? (is-a? event <modeling-event>))
+               (port ((compose .port car ast:trigger*) on))
+               (provides? (and port (ast:provides? port)))
+               (model (ast:parent on <model>))
+               (statement (triple-statement t))
+               (reply?
+                (or (and (is-a? model <interface>)
+                         (not modeling?))
+                    (and provides?
+                         (null? (tree-collect
+                                 (conjoin
+                                  (is? <reply>)
+                                  (disjoin (negate .port)
+                                           (compose (cute ast:eq? port <>)
+                                                    .port))) statement)))))
+               (reply (if reply? (list (make <reply>)) '()))
+               (elements (if (is-a? statement <compound>)
+                             (ast:statement* statement)
+                             (list statement)))
+               (elements (append elements reply))
+               (statement (make <compound> #:elements elements))
+               (t (make-triple on
+                               (triple-guard t)
+                               (triple-blocking? t)
+                               statement)))
+          (add-the-end t))))))
   (map mark-the-end triples))
 
 (define (add-the-end t)
@@ -435,17 +431,12 @@
     (make-triple (triple-on t) (triple-guard t) (triple-blocking? t) statement)))
 
 (define ((triples:declarative-illegals model) triples)
-  (define (illegal? o)
-    (match o
-      (($ <illegal>) o)
-      ((and ($ <compound>) (= ast:statement* (statement))) (illegal? statement))
-      (_ #f)))
   (define (foo t)
     (let* ((on (triple-on t))
            (trigger ((compose car ast:trigger*) on))
            (provides? (and=> (.port trigger) ast:provides?))
            (statement (triple-statement t)))
-      (if (and (or (is-a? model <interface>) provides?) (illegal? statement)) (make-triple on (triple-guard t) (triple-blocking? t) (make <declarative-illegal>))
+      (if (and (or (is-a? model <interface>) provides?) (ast:illegal? statement)) (make-triple on (triple-guard t) (triple-blocking? t) (make <declarative-illegal>))
           t)))
   (map foo triples))
 
@@ -483,14 +474,6 @@
                    (conjoin (is? <ast>) (disjoin ast:declarative? (compose ast:declarative? .parent)))
                    ast:imperative? o))))
 
-(define (triples:on-compound triples)
-  (define (foo t)
-    (let* ((statement (triple-statement t))
-           (statement (if (is-a? statement <compound>) statement
-                          (make <compound> #:elements (list statement)))))
-      (make-triple (triple-on t) (triple-guard t) (triple-blocking? t) statement)))
-  (map foo triples))
-
 (define (normalize:state o)
   "Push guards up, thereby splitting the body of a trigger into multiple
 guarded occurrences."
@@ -502,7 +485,6 @@ guarded occurrences."
               triples:->compound-guard-on
               (cute triples:group-expressions <> (list <and> <field-test> <or>))
               triples:simplify-guard
-              (triples:fix-empty-interface (ast:parent o <model>))
               triples:split-multiple-on
               triples:->triples
               .statement
@@ -524,7 +506,6 @@ guarded occurrences."
               triples:->compound-guard-on
               (cute triples:group-expressions <> (list <and> <field-test> <or>))
               triples:simplify-guard
-              (triples:fix-empty-interface (ast:parent o <model>))
               (triples:add-illegals (ast:parent o <model>))
               triples:mark-the-end
               (triples:declarative-illegals (ast:parent o <model>))
@@ -673,7 +654,7 @@ to prevent unintended shadowing
        (let ((name (if (formal-or-local? o) (rename-string (.name o))
                        (.name o))))
          (clone o #:name name #:expression ((rename mapping) (.expression o)))))
-      (($ <field-test>) (if (formal-or-local? (.variable o)) (clone o #:variable.name (rename-string (.variable.name o)))
+      (($ <field-test>) (if (formal-or-local? (.variable o)) (clone o #:name (rename-string (.variable.name o)))
                             o))
       (($ <formal>) (clone o #:name (rename-string (.name o))))
       (($ <formal-binding>) (clone o #:name (rename-string (.name o))))
@@ -730,12 +711,11 @@ to prevent unintended shadowing
            ((rename mapping) (triple-statement t))))))
   (map foo triples))
 
-(define-method (is-data? (o <ast>))
-  (or (is-a? o <data>) (and (is-a? o <var>) (is-a? ((compose .type .variable) o) <extern>))))
-
 (define (purge-data o)
   "Remove every `extern' data variable and reference."
   (match o
+    (($ <out-bindings>)
+     (clone o #:elements '()))
     ((? (is? <ast-list>))
      (clone o #:elements (filter-map purge-data (.elements o))))
     (($ <data>)
@@ -822,6 +802,71 @@ to prevent unintended shadowing
     ((? (is? <ast>)) (tree-map add-function-return o))
     (_ o)))
 
+(define (extract-call o)
+  "Move typed function calls from variable initialization and assignment
+to a separate statement, for mCRL2."
+  (define (extract-assign/variable-call o call)
+    (let ((expression (make <literal> #:value "return_value")))
+      (match o
+        (($ <assign>)
+         (let ((assign (clone o #:expression expression)))
+           (list call assign)))
+        (($ <variable>)
+         (let* ((default (ast:default-value o))
+                (variable (clone o #:expression default))
+                (assign (make <assign> #:variable.name (.name variable)
+                              #:expression expression)))
+           (list variable call assign))))))
+  (match o
+    (($ <behavior>)
+     (clone o
+            #:functions (extract-call (.functions o))
+            #:statement (extract-call (.statement o))))
+    (($ <defer>)
+     (clone o #:statement (extract-call (.statement o))))
+    (($ <blocking>)
+     (clone o #:statement (extract-call (.statement o))))
+    (($ <on>)
+     (clone o #:statement (extract-call (.statement o))))
+    (($ <guard>)
+     (clone o #:statement (extract-call (.statement o))))
+    (($ <functions>)
+     (clone o #:elements (map extract-call (ast:function* o))))
+    ((and (or ($ <assign>)
+              ($ <variable>))
+          (= .expression (and ($ <call>) call)))
+     (let* ((statements (extract-assign/variable-call o call))
+            (compound (make <compound> #:elements statements)))
+       (clone compound #:parent (.parent o))))
+    (($ <compound>)
+     (let ((statements
+            (let loop ((statements (ast:statement* o)))
+              (match statements
+                (()
+                 '())
+                ((statement rest ...)
+                 (match statement
+                   ((and (or ($ <assign>)
+                             ($ <variable>))
+                         (= .expression (and ($ <call>) call)))
+                    (let ((statements (extract-assign/variable-call
+                                       statement call)))
+                      (append
+                       statements
+                       (loop rest))))
+                   (_
+                    (cons (extract-call statement)
+                          (loop rest)))))))))
+       (clone o #:elements statements)))
+    ((? (%normalize:short-circuit?))
+     o)
+    (($ <interface>)
+     (clone o #:behavior (extract-call (.behavior o))))
+    (($ <component>)
+     (clone o #:behavior (extract-call (.behavior o))))
+    ((? (is? <ast>)) (tree-map extract-call o))
+    (_ o)))
+
 (define* (add-defer-end o)
   (define* (add-end o #:key (loc o))
     (match o
@@ -1316,45 +1361,44 @@ code check."
   (define (location o)
     (or (.location o)
         (location (.parent o))))
-  (define (add-tag o)
-    (match o
-      ((? declarative-illegal?)
-       o)
-      (($ <compound>)
-       (let ((tag (make <tag> #:location (location o))))
-         (clone o #:elements (cons tag (ast:statement* o)))))
-      (_
-       (let* ((location (location o))
-              (tag (make <tag> #:location location)))
-         (make <compound> #:elements (list tag o) #:location location)))))
+  (define (add-tag-imperative o)
+    (if (ast:imperative? o)
+        (match o
+          ((? ast:illegal?)
+           o)
+          (($ <compound>)
+           (let ((tag (make <tag> #:location (location o))))
+             (clone o #:elements (cons tag (ast:statement* o)))))
+          (_
+           (let* ((location (location o))
+                  (tag (make <tag> #:location location)))
+             (make <compound> #:elements (list tag o) #:location location))))
+        o))
 
   (match o
-    (($ <function>)
+    ((or ($ <blocking>) ($ <defer>) ($ <function>) ($ <guard>) ($ <on>))
      (let* ((statement (.statement o))
-            (statement (add-tag (tag-imperative-blocks statement))))
-       (clone o #:statement statement)))
-    (($ <defer>)
-     (let* ((statement (.statement o))
-            (statement (add-tag (tag-imperative-blocks statement))))
+            (statement (add-tag-imperative (tag-imperative-blocks statement))))
        (clone o #:statement statement)))
     (($ <if>)
-     (let ((then (add-tag (tag-imperative-blocks (.then o))))
-           (else (and=> (tag-imperative-blocks (.else o)) add-tag)))
+     (let ((then (add-tag-imperative (tag-imperative-blocks (.then o))))
+           (else (and=> (tag-imperative-blocks (.else o)) add-tag-imperative)))
        (clone o #:then then #:else else)))
-    (($ <variables>)
+    (($ <compound>)
+     (clone o #:elements (map tag-imperative-blocks (ast:statement* o))))
+    ((? (is? <statement>))
      o)
-    (($ <illegal>)
-     o)
-    ((? ast:imperative?)
-     (let* ((o (tree-map tag-imperative-blocks o))
-            (p (.parent o))
-            (add-tag? (and (is-a? p <statement>)
-                           (not (ast:imperative? p)))))
-       (if (not add-tag?) o
-           (add-tag o))))
+    (($ <behavior>)
+     (clone o #:statement (tag-imperative-blocks (.statement o))
+            #:functions (tag-imperative-blocks (.functions o))))
     ((? (%normalize:short-circuit?))
      o)
+    (($ <component>)
+     (clone o #:behavior (tag-imperative-blocks (.behavior o))))
+    (($ <interface>)
+     (clone o #:behavior (tag-imperative-blocks (.behavior o))))
+    ((? (is? <type>))
+     o)
     ((? (is? <ast>))
      (tree-map tag-imperative-blocks o))
-    (_
-     o)))
+    (_ o)))
