@@ -1,8 +1,8 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2019, 2020, 2021, 2022, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2019, 2020, 2021, 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
-;;; Copyright © 2019, 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2019, 2020, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2021, 2023 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2019 Johri van Eerd <vaneerd.johri@gmail.com>
 ;;;
@@ -35,7 +35,6 @@
 
   #:use-module (dzn ast accessor)
   #:use-module (dzn ast goops)
-  #:use-module (dzn ast silence)
   #:use-module (dzn ast util)
   #:use-module (dzn ast)
   #:use-module (dzn misc)
@@ -46,20 +45,16 @@
   (define (make-list? o) (if (pair? o) o
                              (list o)))
 
-  (define (text->lines-offsets string)
-    "Return a vector of pairs from STRING
-
-    #((0 . end-1) (start-2 . end-2) (start-n . end-n))
-
-where start-<n> is the position of newline."
-    (let ((lines (let loop ((start 0))
-                   (let ((pos (string-index string #\newline start)))
-                     (if (not pos) `((,start . ,start))
-                         (cons `(,start . ,pos)
-                               (loop (1+ pos))))))))
-      (list->vector lines)))
-
-  (define newlines (text->lines-offsets string))
+  (define newlines ;; --> (vector (pos1 . pos2) (pos2+1 . pos3) ...) where posi is position of newline
+    (let* ((last (string-length string))
+           (lines (let loop ((current-pos 0) (result '()))
+                    (let ((index (string-index string #\newline current-pos last)))
+                      (if (not index) result
+                          (loop (1+ index) (append result (list index)))))))
+           (lines (let loop ((start 0) (todo lines) (result '()))
+                    (if (null? todo) (cons (cons start last) result)
+                        (loop (1+ (car todo)) (cdr todo) (cons (cons start (car todo)) result))))))
+      (list->vector (reverse lines))))
 
   (define (compare-pos pos nl)
     (cond ((< (cdr pos) (car nl)) -1)
@@ -73,6 +68,20 @@ where start-<n> is the position of newline."
   (define (column-number line pos)
     (1+ (- pos (car (vector-ref newlines (1- line))))))
 
+  (define (debug-location ast)
+    "Usage (pke (debug-locatation o)) while constructing AST"
+    (define (print-location o)
+      (match o
+        (('location pos end)
+         (let* ((line (line-number pos))
+                (column (1- (column-number line pos)))
+                (end-line (line-number end))
+                (end-column (column-number end-line end)))
+           (simple-format #f "~a: ~a:~a-~a:~a\n" ast line column end-line end-column)))
+        ((type children ...) (map print-location children))
+        (_ o)))
+    (print-location ast))
+
   (define (file-helper o file-name start-pos)
 
     (define (make-interface name types-and-events behavior comment)
@@ -95,7 +104,7 @@ where start-<n> is the position of newline."
                            (_
                             events))))
              (events (make <events-node> #:elements events))
-             (behavior (and behavior (annotate-functions (helper behavior)))))
+             (behavior (and behavior (helper behavior))))
         (make <interface-node>
           #:name (helper name)
           #:comment comment
@@ -284,7 +293,7 @@ to the AST element."
            #:comment comment
            #:name (helper name)
            #:ports (helper ports)
-           #:behavior (annotate-functions (helper behavior))))
+           #:behavior (helper behavior)))
 
         (('component name ports ('system instances-and-bindings rest ...))
          (let* ((instances-and-bindings (helper instances-and-bindings))
@@ -363,9 +372,7 @@ to the AST element."
 
         (('compound body ...)
          (let ((body (helper body)))
-           (make (if (is-a? (car body) <declarative>) <declarative-compound-node>
-                     <compound-node>)
-             #:elements body)))
+           (make <compound-node> #:elements body)))
 
         (('on triggers)
          (make <on-node> #:comment comment #:triggers (helper triggers)))
@@ -411,6 +418,9 @@ to the AST element."
         (('compound-name global scope name)
          (make <scope.name-node> #:ids (append (helper global) (helper scope) (list (helper name)))))
 
+        (('scoped-name name)
+         (make <scope.name-node> #:ids (list (helper name))))
+
         (('scope name) (make-list? (helper name)))
         (('scope names ...) (helper names))
 
@@ -507,8 +517,10 @@ to the AST element."
 
         (('dollars) (make <data-node> #:value *unspecified*))
 
-        (('field-test ('var name _ ...) field) (make <field-test-node> #:variable.name (helper name) #:field (helper field)))
-        (('field-test ('unknown-identifier identifier _ ...) field) (make <field-test-node> #:variable.name identifier #:field (helper field)))
+        (('field-test ('var name _ ...) field) (make <field-test-node> #:name (helper name) #:field (helper field)))
+        (('field-test ('unknown-identifier identifier _ ...) field) (make <field-test-node> #:name identifier #:field (helper field)))
+
+        (('shared-field-test port.name name field) (make <shared-field-test-node> #:port.name (helper port.name) #:name (helper name) #:field (helper field)))
 
         (('function type name formals statement)
          (let* ((type location comment (helper type))
@@ -604,6 +616,8 @@ to the AST element."
 
         (('var name) (make <var-node> #:name (helper name)))
 
+        (('shared-var port.name name) (make <shared-var-node> #:port.name (helper port.name) #:name (helper name)))
+
         (('variable type name)
          (make <variable-node> #:name (helper name) #:type.name (helper type) #:expression (make <literal-node>)))
 
@@ -653,40 +667,5 @@ to the AST element."
 
     (helper o))
 
-  (let* ((root-node (file-helper o file-name 0))
-         (root (make <root> #:node root-node)))
-    (tree-map make-namespaces root)))
-
-(define-method (set-recursive (o <behavior>))
-  (define (mark-recursive f)
-    (if (ast:recursive? f) (clone f #:recursive? #t)
-        f))
-  (let* ((functions (.functions o))
-         (function-list (.elements functions))
-         (function-list (map mark-recursive function-list))
-         (functions (clone functions #:elements function-list)))
-    (clone o #:functions functions)))
-
-(define-method (set-noisy (o <behavior>))
-  (mark-noisy o))
-
-(define-method (annotate-functions (o <behavior-node>))
-  (let* ((o (set-recursive (make <behavior> #:node o)))
-         (o (set-noisy o)))
-    (.node o)))
-
-(define-method (make-namespaces (o <ast>))
-  o)
-
-(define-method (make-namespaces (o <model>))
-  (let ((scope (ast:scope o)))
-    (let loop ((scope scope))
-      (if (null? scope) o
-          (make <namespace> #:name (make <scope.name> #:ids (list (car scope))) #:elements (list (loop (cdr scope))))))))
-
-(define-method (make-namespaces (o <type>))
-  (if (ast:parent o <model>) o
-      (let ((scope (ast:scope o)))
-        (let loop ((scope scope))
-          (if (null? scope) o
-              (make <namespace> #:name (make <scope.name> #:ids (list (car scope))) #:elements (list (loop (cdr scope)))))))))
+  (let ((root-node (file-helper o file-name 0)))
+    (make <root> #:node root-node)))
