@@ -1,9 +1,9 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2017, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2014, 2017, 2020, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
-;;; Copyright © 2020, 2021, 2022 Paul Hoogendijk <paul@dezyne.org>
+;;; Copyright © 2020, 2021, 2022, 2023 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -26,12 +26,13 @@
 
   #:use-module (ice-9 curried-definitions)
   #:use-module (ice-9 match)
-  #:use-module (ice-9 optargs)
 
   #:use-module (dzn ast equal)
   #:use-module (dzn ast goops)
   #:use-module (dzn ast lookup)
   #:use-module (dzn ast)
+  #:use-module (dzn code)
+  #:use-module (dzn config)
   #:use-module (dzn misc)
 
   #:export (ast:wfc
@@ -156,11 +157,22 @@
       (and=> (.behavior o) wfc)
       '()))
 
+(define-method (wfc-external (o <port>))
+  (let* ((model (ast:parent o <model>))
+         (shell? (and (%shell)
+                      (member (ast:dotted-name model) (%shell)))))
+    (if (or (not shell?) (ast:external? o)) '()
+        `(,(wfc-error
+            o
+            (format #f "outer shell port `~a' must be declared external"
+                    (.name o)))))))
+
 (define-method (wfc-skeleton (o <system>))
   (append
    (re-definition o)
    (append-map wfc (ast:port* o))
-   (append-map wfc (ast:instance* o))))
+   (append-map wfc (ast:instance* o))
+   (append-map wfc-external (ast:requires-port* o))))
 
 (define-method (wfc (o <system>))
   (or (as (wfc-skeleton o) <pair>)
@@ -190,25 +202,37 @@
   (re-definition o))
 
 (define-method (wfc (o <enum>))
+  (define (fields-unique fields)
+    (if (null? fields) '()
+        (let* ((field (car fields))
+               (dupe (find (cut equal? <> field) (cdr fields)))
+               (errors (if (not dupe) '()
+                           `(,(wfc-error
+                               o
+                               (format #f "duplicate enum field `~a' in enum `~a'"
+                                       field (type-name (.name o))))))))
+          (append errors (fields-unique (cdr fields))))))
   (append
    (re-definition o)
-   (let loop ((fields (ast:field* o)) (result '()))
-     (if (null? fields) result
-         (let* ((field (car fields))
-                (wf (if (find (cut equal? <> field) (cdr fields))
-                        `(,(wfc-error o (format #f "duplicate enum field `~a' in enum `~a'" field (type-name (.name o)))))
-                        '())))
-           (loop (cdr fields) (append wf result)))))
-   (if (and (ast:parent o <model>) (ast:name-equal? (.name (ast:parent o <model>)) (.name o)))
-       `(,(wfc-error o (format #f "enum `~a' must not have the same name as the model it is defined in" (type-name (.name o)))))
+   (fields-unique (ast:field* o))
+   (if (and (ast:parent o <model>)
+            (ast:name-equal? (.name (ast:parent o <model>)) (.name o)))
+       `(,(wfc-error
+           o
+           (format
+            #f
+            "enum `~a' must not have the same name as the model it is defined in"
+            (type-name (.name o)))))
        '())
    '()))
 
 (define-method (wfc (o <subint>))
-  (append (re-definition o)
-          (let ((range (.range o)))
-            (if (<= (.from range) (.to range)) '()
-                `(,(wfc-error o (format #f "subint `~a' has empty range" (type-name (.name o)))))))))
+  (append
+   (re-definition o)
+   (let ((range (.range o)))
+     (if (<= (.from range) (.to range)) '()
+         `(,(wfc-error o (format #f "subint `~a' has empty range"
+                                 (type-name (.name o)))))))))
 
 (define-method (wfc (o <port>))
   (append
@@ -405,12 +429,54 @@
    (re-definition o)
    (assign o)))
 
+(define-method (wfc-constraint (o <variable>) (port <port>))
+  (append
+   (if (feature-supported? "shared") '()
+       `(,(wfc-error
+           o
+           (format #f "using shared variable `~a.~a' with language ~s is not supported"
+                   (.name port)
+                   (.name o)
+                   (%language)))))
+   (if (not (%no-constraint?)) '()
+       `(,(wfc-error
+           o
+           (format #f "using shared variable `~a.~a' with --no-constraint"
+                   (.name port)
+                   (.name o)))))
+   (if (not (ast:external? port)) '()
+       `(,(wfc-error
+           o
+           (format #f "using shared variable `~a.~a' with external port `~a'"
+                   (.name port)
+                   (.name o)
+                   (.name port)))))))
+
+(define-method (wfc-constraint (o <top>) (port <top>))
+  '())
+
+(define-method (wfc-constraint (o <shared-variable>))
+  (wfc-constraint o (.port o)))
+
+(define-method (wfc-port (o <ast>))
+  (let ((port (.port o)))
+    (if (is-a? port <port>) '()
+        `(,(wfc-error o (format #f "undefined port `~a'" (.port.name o)))))))
+
+(define-method (wfc (o <shared-variable>))
+  (append
+   (wfc-port o)
+   (next-method)
+   (wfc-constraint o)))
+
 (define-method (wfc (model <model>) (o <variable>))
   (if (ast:name-equal? (.name model) (.name o))
-      `(,(wfc-error o
-                    (format #f
-                            "variable `~a' must not have the same name as the model it is defined in"
-                            (.name o))))
+      `(,(wfc-error
+          o
+          (format
+           #f
+           "variable `~a' must not have the same name as the model it is defined in"
+           (.name o))))
       '()))
 
 (define-method (wfc (o <function>))
@@ -467,6 +533,8 @@
             (if (or otherwise? (pair? wfce)) '()
                 (typed-expression expression <bool>))
             (if otherwise? (otherwise o) '())
+            (if (not (ast:parent o <imperative>)) '()
+                `(,(wfc-error o "cannot use a guard in imperative context")))
             (if (is-a? (.statement o) <statement>) (wfc (.statement o))
                 `(,(wfc-error o "statement expected"))))))
 
@@ -483,7 +551,14 @@
              `(,(wfc-error o "nested blocking used")
                ,(wfc-info (ast:parent o <blocking>) "within blocking here")))
             (else '()))))
-  (append (blocking o) (wfc (.statement o))))
+  (append
+   (if (feature-supported? "block") '()
+       `(,(wfc-error
+           o
+           (format #f "use of `blocking' with language ~s is not supported"
+                   (%language)))))
+   (blocking o)
+   (wfc (.statement o))))
 
 (define (wfc-defer-argument var)
   (if (is-a? var <undefined>)
@@ -492,6 +567,12 @@
              (name (.variable.name var))
              (type (and=> variable .type)))
         (cond
+         ((is-a? var <shared>)
+          `(,(wfc-error
+              var
+              (format #f "cannot use interface variable `~a' as defer argument"
+                      name))
+            ,(wfc-info variable (format #f "variable `~a' defined here" name))))
          ((not (ast:member? variable))
           `(,(wfc-error var (format
                              #f
@@ -510,6 +591,11 @@
 (define-method (wfc (o <defer>))
   (let ((arguments (ast:argument* o)))
     (append
+     (if (feature-supported? "defer") '()
+         `(,(wfc-error
+             o
+             (format #f "use of `blocking' with language ~s is not supported"
+                     (%language)))))
      (if (not arguments) '()
          (append-map wfc-defer-argument arguments))
      (wfc (.statement o)))))
@@ -521,11 +607,27 @@
        (if parent `(,(wfc-error o "nested on used")
                     ,(wfc-info parent "within on here"))
            '()))))
-  (append
-   (on o)
-   (append-map wfc (ast:trigger* o))
-   (if (is-a? (.statement o) <statement>) (wfc (.statement o))
-       `(,(wfc-error o "statement expected")))))
+  (define (triggers-unique triggers)
+    (if (null? triggers) '()
+        (let* ((trigger (car triggers))
+               (dupe (find (cute ast:equal? <> trigger) (cdr triggers)))
+               (errors (if (not dupe) '()
+                           `(,(wfc-error
+                               dupe
+                               (format #f "duplicate trigger `~a' in on"
+                                       (trigger->string trigger)))
+                             ,(wfc-info
+                               trigger
+                               (format #f "previous `~a' trigger here"
+                                       (trigger->string trigger)))))))
+          (append errors (triggers-unique (cdr triggers))))))
+  (let ((triggers (ast:trigger* o)))
+    (append
+     (on o)
+     (triggers-unique triggers)
+     (append-map wfc triggers)
+     (if (is-a? (.statement o) <statement>) (wfc (.statement o))
+         `(,(wfc-error o "statement expected"))))))
 
 (define-method (wfc (o <imperative>))
   '())
@@ -598,7 +700,7 @@
   (let ((port (.port o))
         (on (ast:parent o <on>)))
     (cond ((not port)
-           `(,(wfc-error o (format #f "undefined port `~a'" (.port.name o)))))
+           (wfc-port o))
           ((not (is-a? (.type port) <interface>)) '()) ; reported before
           ((ast:requires? port)
            `(,(wfc-error o (format #f "cannot use requires port `~a' in reply"
@@ -609,8 +711,6 @@
 
 (define-method (reply-without-port (o <reply>))
   "pre: (not (.port.name o))"
-  (define (trigger->string o)
-    (format #f "~a.~a" (.port.name o) (.event.name o)))
   (let ((model (ast:parent o <model>))
         (on (ast:parent o <on>)))
     (cond ((or (not (is-a? model <component>))
@@ -650,10 +750,19 @@
                                     reply-type-name)))))))))
 
 (define-method (wfc (o <return>))
+  (define (outer-compound o)
+    (let ((parent (.parent o)))
+      (if (is-a? parent <compound>) (outer-compound parent) o)))
+  (define (last-statement o)
+    (if (is-a? o <compound>)
+        (let ((statements (.elements o)))
+          (if (pair? statements) (last statements) o))
+        o))
   (let* ((wfce (if (.expression o) (wfc (.expression o)) '()))
          (function (ast:parent o <function>))
          (function-type (and function (ast:type function)))
-         (return-type (and (null? wfce) (ast:type o))))
+         (return-type (and (null? wfce) (ast:type o)))
+         (last-statement-block (last-statement (outer-compound o))))
     (append wfce
             (cond ((not function)
                    `(,(wfc-error o "cannot use return outside of function")))
@@ -662,6 +771,9 @@
                    `(,(wfc-error o (format #f "type mismatch: expected `~a', found: `~a'"
                                            (type-name function-type)
                                            (type-name return-type)))))
+                  ((not (ast:eq? last-statement-block o))
+                   `(,(wfc-error ((compose car ast:continuation*) o)
+                                 (format #f "code will never be executed"))))
                   (else '())))))
 
 (define-method (wfc (o <the-end>))
@@ -676,7 +788,7 @@
         (model (ast:parent o <model>)))
     (append
      (cond ((and (is-a? model <component>) (not port))
-            `(,(wfc-error o (format #f "undefined port `~a'" (.port.name o)))))
+            (wfc-port o))
            ((and (is-a? model <component>) (not (is-a? port <port>)))
             `(,(wfc-error o (format #f "`~a' is not a port" (.port.name o)))
               ,(wfc-info (.port o) (format #f "`~a' declared here" (.port.name o)))))
@@ -818,6 +930,12 @@
              ,(wfc-info type "enum defined here")))
           (else '()))))
 
+(define-method (wfc (o <shared-field-test>))
+  (append
+   (next-method)
+   (wfc-port o)
+   (wfc-constraint (.variable o) (.port o))))
+
 (define-method (wfc (o <otherwise>)) '())
 
 (define-method (wfc (o <var>))
@@ -831,10 +949,26 @@
             `(,(wfc-error o (format #f "undefined variable `~a'" (.name o)))))
            (else '())))))
 
+(define-method (wfc (o <shared-var>))
+  (append
+   (next-method)
+   (wfc-port o)
+   (wfc-constraint (.variable o) (.port o))))
+
 (define-method (wfc (o <undefined>))
   `(,(wfc-error o (format #f "undefined identifier `~a'" (.name o)))))
 
-(define-method (wfc (o <data>)) '())
+(define-method (wfc (o <data>))
+  (let* ((value (.value o))
+         (supported? (or (member (%language) '("dzn" "makreel"))
+                         (not (unspecified? value)))))
+    (if supported? '()
+        `(,(wfc-error o (format #f "Unspecified dollar escaped data"))))))
+
+(define-method (wfc (o <extern>))
+  (append
+   (next-method)
+   (wfc (.value o))))
 
 (define-method (wfc (o <ast>))
   ;;  (warn 'wfc:--------------UNCOVERED-------------- o)
@@ -849,28 +983,38 @@
             (ast:function* (ast:parent o <behavior>))) '()
             `(,(wfc-error o (format #f "undefined function call: ~s" (.function.name o))))))
 
+(define (feature-supported? feature)
+  (or (member (%language) '("dzn" "makreel"))
+      (member (%language) (assoc-ref %feature-alist feature))))
+
 (define (single-quote string)
   (format #f "`~a'" string))
 
+(define (trigger->string o)
+  (format #f "~a.~a" (.port.name o) (.event.name o)))
+
 (define-method (tail-recursion (o <call>))
-  (let ((function (ast:parent o <function>)))
-    (if (or (not function) (not (.recursive? function))) '()
-        (let* ((continuation ((compose car wfc:continuation) o))
-               (continuation (if (or (ast:parent o <assign>)
-                                     (ast:parent o <reply>)
-                                     (ast:parent o <return>)
-                                     (ast:parent o <variable>))
-                                 ((compose car (@@ (dzn code makreel) makreel:continuation)) continuation)
-                                 continuation))
-               (continuation (and continuation
-                                  (not (ast:eq? continuation (.statement (ast:parent o <function>))))
+  (let* ((function (ast:parent o <function>))
+         (called (.function o))
+         (non-recursive? (or (not function) (not (.recursive? function))
+                             (not called) (not (.recursive? called)))))
+    (if non-recursive? '()
+        (let* ((assign (or (ast:parent o <assign>)
+                           (ast:parent o <return>)
+                           (ast:parent o <variable>)))
+               (continuation ((compose car ast:continuation*) o))
+               (statement (.statement function))
+               (continuation (and (not (ast:eq? continuation statement))
+                                  (not (is-a? continuation <return>))
                                   continuation)))
-          (cond ((is-a? continuation <return>)
-                 `(,(wfc-error o "cannot use valued function in recursion")
-                   ,(wfc-info continuation "statement after call")))
-                (continuation `(,(wfc-error o "cannot use statement after recursive call")
-                                ,(wfc-info continuation "statement after call")))
-                (else '()))))))
+          (cond
+           (assign
+            `(,(wfc-error o "cannot use typed function in recursion")
+              ,(wfc-info assign "statement after call")))
+           (continuation
+            `(,(wfc-error o "cannot use statement after recursive call")
+              ,(wfc-info continuation "statement after call")))
+           (else '()))))))
 
 (define-method (mixing-declarative-imperative (o <compound>))
   (if (ast:declarative? o)
@@ -1213,7 +1357,7 @@
      (else
       (run continuation))))
   (define (run o)
-    (let ((continuations (wfc:continuation o)))
+    (let ((continuations (ast:continuation* o)))
       (if (is-a? o <if>)
           (append-map (cute step o <>) continuations)
           (step o (car continuations)))))
@@ -1247,7 +1391,7 @@
     (cond
      ((and (is-a? o <action>) (not (.event o)))
       (let ((name (.event.name o)))
-        `(,(wfc-error o (format #f "undefined identifier `~a'" name)))))
+        `(,(wfc-error o (format #f "undefined event `~a'" name)))))
      ((and (ast:parent o <variable>)
            (ast:member? (ast:parent o <variable>)))
       (let ((class (if (equal? class "var") "variable reference" class)))
@@ -1283,13 +1427,6 @@
         `(,(wfc-error o (format #f "~a outside on" class)))))
      (else '()))))
 
-(define-method (wfc:continuation (o <ast>))
-  ((@@ (dzn code makreel) makreel:continuation) o))
-
-(define-method (wfc:continuation (o <if>))
-  (append ((@@ (dzn code makreel) makreel:then-continuation) o)
-          ((@@ (dzn code makreel) makreel:else-continuation) o)))
-
 (define-method (recursive? (o <system>))
   (ast:graph-cyclic? ast:system* o))
 
@@ -1328,10 +1465,11 @@
 (define-method (cyclic-bindings (o <system>))
   (let* ((instances (ast:instance* o))
          (required-alist (map (cut requires-instances <> o) instances)))
-    (append-map (lambda (i) (if (find (cut ast:eq? i <>) (all-required i o required-alist))
-                                `(,(wfc-error i (format #f "instance `~a' is in a cyclic binding" (.name i))))
-                                '()))
-                instances)))
+    (define (check-instance i)
+      (if (not (find (cute ast:eq? <> i) (all-required i o required-alist))) '()
+          `(,(wfc-error
+              i (format #f "instance `~a' is in a cyclic binding" (.name i))))))
+    (append-map check-instance instances)))
 
 (define-method (wfc:provides-in-event* (o <component>))
   (let* ((ports (ast:provides-port* o))
@@ -1366,5 +1504,5 @@
 ;;;
 (define-method (wfc (o <root>))
   (append
-   (append-map wfc (ast:model* o))
-   (append-map wfc (ast:type* o))))
+   (append-map wfc (ast:model** o))
+   (append-map wfc (ast:type** o))))
