@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2021, 2023 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
@@ -24,19 +24,19 @@
 ;;; Code:
 
 (define-module (dzn commands graph)
-  #:use-module (ice-9 getopt-long)
   #:use-module (srfi srfi-26)
 
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
-  #:use-module (dzn goops)
-  #:use-module (dzn command-line)
-  #:use-module (dzn explore)
+  #:use-module (ice-9 getopt-long)
+
   #:use-module (dzn ast)
   #:use-module (dzn code)
-  #:use-module (dzn config)
-  #:use-module (dzn parse)
+  #:use-module (dzn command-line)
   #:use-module (dzn commands parse)
-  #:use-module (dzn vm normalize)
+  #:use-module (dzn code)
+  #:use-module (dzn config)
+  #:use-module (dzn explore)
+  #:use-module (dzn parse)
+
   #:export (parse-opts
             main))
 
@@ -53,9 +53,9 @@
             (queue-size-defer (value #t))
             (queue-size-external (value #t))
             (remove (single-char #\R) (value #t))))
-	 (options (getopt-long args option-spec))
-	 (help? (option-ref options 'help #f))
-	 (files (option-ref options '() '()))
+         (options (getopt-long args option-spec))
+         (help? (option-ref options 'help #f))
+         (files (option-ref options '() '()))
          (usage? (and (not help?) (null? files))))
     (when (or help? usage?)
       (let ((port (if usage? (current-error-port) (current-output-port))))
@@ -107,10 +107,10 @@ Generate graph from a Dezyne model
          ;; Parse --model=MODEL cuts MODEL from AST; avoid that
          (parse-options (filter (negate (compose (cute eq? <> 'model) car))
                                 options))
-         (ast (parse parse-options file-name))
-         (root (if (member backend '("lts" "state")) (vm:normalize ast) ast))
+         (ast (parameterize ((%language "makreel"))
+                (parse parse-options file-name)))
          (model (call-with-handle-exceptions
-                 (lambda _ (ast:get-model root model-name))
+                 (lambda _ (ast:get-model ast model-name))
                  #:backtrace? debug?
                  #:file-name file-name))
          (language (option-ref options 'format "dot"))
@@ -128,16 +128,16 @@ Generate graph from a Dezyne model
     (unless model
       (format (current-error-port) "~a: No dezyne model found.\n" file-name)
       (exit EXIT_OTHER_FAILURE))
-    (cond (dependency? (code root
+    (cond (dependency? (code ast
                              #:ast-> 'dependency-diagram
                              #:model model
                              #:language language))
-          (lts? (lts root
+          (lts? (lts ast
                      #:model model
                      #:queue-size queue-size
                      #:queue-size-defer queue-size-defer
                      #:queue-size-external queue-size-external))
-          (state? (state-diagram root
+          (state? (state-diagram ast
                                  #:format language
                                  #:model model
                                  #:queue-size queue-size
@@ -148,8 +148,9 @@ Generate graph from a Dezyne model
                                  #:actions? actions?
                                  #:labels? labels?
                                  #:returns? returns?))
-          (else (code root
+          (else (code ast
                       #:ast-> 'system-diagram
+                      #:dir "-"
                       #:model model
                       #:language language
                       #:locations? locations?)))))
