@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2017, 2018, 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2017, 2018, 2019, 2020, 2021, 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2022, 2023 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 ;;;
@@ -61,23 +61,24 @@
     ((dzn ast normalize) normalize:state+illegals)
     ((dzn code) code:normalize)
     ((dzn code) code:normalize+determinism)
-    ((dzn code c++) ast->)
-    ((dzn code makreel) makreel:normalize)
-    ((dzn code makreel) ast->)
+    ((dzn code language c++) ast->)
+    ((dzn code language makreel) makreel:normalize)
+    ((dzn code scmackerel makreel) root->scmackerel)
+    ((dzn code scmackerel makreel) scmackerel:display)
     ((dzn ast wfc) wfc (,<root>))))
 
 ;; Procedures to instrument with measure-duration.
 (define %measure
   `(((dzn parse) parse-tree->ast)
     ((dzn parse) string->parse-tree)
-    ((dzn code makreel) root->)
+    ((dzn code language makreel) root->)
     ((dzn ast accessor) ast:parent (,<ast> ,<class>))
     ((dzn ast lookup) ast:lookup (,<ast> ,<top>))
     ((dzn ast lookup) ast:lookup-variable (,<ast> ,<top>))))
 
 (define (parse-opts args)
   (let* ((option-spec
-	  '((debug (single-char #\d))
+          '((debug (single-char #\d))
             (help (single-char #\h))
             (repl (single-char #\R))
             (skip-wfc (single-char #\p))
