@@ -3,8 +3,8 @@
 ;;; Copyright © 2016, 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018 Henk Katerberg <hank@mudball.nl>
-;;; Copyright © 2018, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
-;;; Copyright © 2018, 2020, 2021, 2023 Paul Hoogendijk <paul@dezyne.org>
+;;; Copyright © 2018, 2021, 2022, 2023 Rutger (regtur) van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2018, 2020, 2021, 2022, 2023 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2017, 2018 Johri van Eerd <vaneerd.johri@gmail.com>
 ;;;
 ;;; This file is part of Dezyne.
@@ -23,26 +23,24 @@
 ;;; License along with Dezyne.  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (dzn verify pipeline)
-  #:use-module (srfi srfi-9 gnu)
-  #:use-module (ice-9 curried-definitions)
-  #:use-module (ice-9 getopt-long)
-  #:use-module (ice-9 match)
-  #:use-module (ice-9 poe)
-  #:use-module (ice-9 rdelim)
   #:use-module (srfi srfi-1)
+  #:use-module (srfi srfi-9 gnu)
   #:use-module (srfi srfi-26)
   #:use-module (srfi srfi-71)
 
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
+  #:use-module (ice-9 curried-definitions)
+  #:use-module (ice-9 match)
+  #:use-module (ice-9 poe)
+  #:use-module (ice-9 rdelim)
 
+  #:use-module (dzn ast goops)
   #:use-module (dzn ast)
+  #:use-module (dzn code)
+  #:use-module (dzn code language makreel)
   #:use-module (dzn command-line)
   #:use-module (dzn config)
-  #:use-module (dzn goops)
-  #:use-module (dzn code makreel)
   #:use-module (dzn lts)
   #:use-module (dzn misc)
-  #:use-module (dzn shell-util)
   #:use-module (dzn pipe)
 
   #:export (verification:formats
@@ -55,7 +53,7 @@
 ;;; '(dzn verify pipeline)' implements a mCRL2-base pipeline for
 ;;; verification of Dezyne models.  Entry point: dzn verify.
 ;;;
-;;; Starting point is from dzn code -l makreel, (dzn code makreel),
+;;; Starting point is from dzn code -l makreel, (dzn code language makreel),
 ;;; the verification pipeline consists of mCRL2 commands and dzn lts.
 ;;; The result is reported in plain text.
 
@@ -66,12 +64,12 @@
 ;;;
 
 (define (interface-taus model)
-  (let ((alphabet '("inevitable" "optional")))
+  (let ((alphabet '("inevitable" "optional" "tag")))
     (string-join alphabet ",")))
 
 (define (component-taus model)
-  (let ((ports (ast:requires+async-port* model)))
-    (string-join (map makreel:.name ports) ",")))
+  (let ((ports (ast:requires-port* model)))
+    (string-join (cons "tag" (map makreel:.name ports)) ",")))
 
 (define (component-exclude-taus model)
   (let ((ports (ast:requires-port* model)))
@@ -94,7 +92,7 @@ EVENT."
          '("false" "true"))
         (($ <enum>)
          (map enum-literal->event (makreel:enum-fields type)))
-        (($ <int>)
+        (($ <subint>)
          (let* ((range (.range type))
                 (from (.from range)))
            (map number->string (iota (1+ (- (.to range) from)) from))))
@@ -131,15 +129,18 @@ actions."
          (taus (delete-duplicates
                 (append-map events-trigger/action
                             (append out-triggers in-actions))))
-         (taus (cons "<defer>" taus)))
+         (state-taus (map (compose (cute string-append <> ".<state>")
+                                   makreel:.name)
+                          (ast:port* model)))
+         (taus `("tag" "<defer>" ,@state-taus ,@taus)))
     (string-join taus ",")))
 
 (define (deterministic-labels component)
   (define (trigger->event trigger)
     (let ((port (.port trigger)))
       (string-append (makreel:.name (.port trigger))
-                     (if (or (ast:async? trigger) (ast:out? trigger)) ".qout."
-                         ".")
+                     (if  (ast:out? trigger) ".qout."
+                          ".")
                      (.event.name trigger))))
   (let* ((triggers (ast:in-triggers component))
          (alphabet (map trigger->event triggers)))
@@ -149,11 +150,9 @@ actions."
   (let ((trace (string-map (lambda (c) (if (eq? c #\newline) #\; c)) trace)))
     (string-join
      (filter (lambda (event)
-               (and (not (member event '("inevitable" "optional" "tau" ;;"<illegal>" "<declarative-illegal>"
-                                         )))
-                    (not (string-contains event ".qout."))
-                    (not (string-contains event ".<blocking>"))
-                    (not (find (cute string-suffix? <> event) '(".optional" ".inevitable")))))
+               (not (or (member event '("inevitable" "optional" "tau"))
+                        (find (cute string-contains event <>) '(".qout." ".<blocking>" "tag(" "<state>"))
+                        (find (cute string-suffix? <> event) '(".optional" ".inevitable")))))
              (string-split trace #\;))
      "\n")))
 
@@ -165,17 +164,32 @@ actions."
              (string-split trace #\;))
      "\n")))
 
+(define (model-tags model)
+  (let* ((tags (tree-collect-filter (negate (disjoin (is? <expression>)
+                                                     (is? <location>)))
+                                    (is? <tag>)
+                                    model))
+         (tags (map makreel:line-column tags)))
+    (delete-duplicates tags)))
+
+(define (makreel:model-tags model)
+  (define (line-column o)
+    (format #f "tag(~a)" o))
+  (let ((tags (model-tags model)))
+    (map (cute format #f "tag(~a)" <>) tags)))
+
 
 ;;;
 ;;; Verify pipeline.
 ;;;
 
 (define-immutable-record-type <options>
-  (make-options root model init)
+  (make-options root model init tags)
   options?
   (root options-root)
   (model options-model)
-  (init options-init))
+  (init options-init)
+  (tags options-tags))
 
 (define (get-commands in-out.pipeline format) ;target-format -> commands
   (define (get-input in-out.pipeline format) ;target-format -> input
@@ -196,7 +210,7 @@ actions."
                  (loop (get-input in-out.pipeline format))))))))
 
 (define root+model->makreel
-  (pure-funcq
+  (perfect-funcq 1024
    (lambda (root model)
      (with-output-to-string (cute makreel:model->makreel root model)))))
 
@@ -209,6 +223,9 @@ actions."
       (newline)
       (display (makreel:init-process (options-init options))))))
 
+(define (component-stage)
+  "verify-component")
+
 (define (in-out:dzn->aut+provides-aut options)
   (let* ((model (options-model options))
          (root (options-root options))
@@ -219,7 +236,7 @@ actions."
            ;; "aut-failures" pipeline.  Running the already memoized
            ;; "verify-component" gives us the same result, and has
            ;; already been memoized.
-           (get-lts (result-split (verify-pipeline "verify-component" root model)))
+           (get-lts (result-split (verify-pipeline (component-stage) root model)))
            "\n\x04\n"
            (verify-pipeline "aut-failures" root model #:init provides-init)))))
 
@@ -228,6 +245,11 @@ actions."
          (root (options-root options)))
     (cute display (verify-pipeline "aut-dpweak-bisim" root model))))
 
+(define (in-out:dzn->aut-cached options)
+  (let* ((model (options-model options))
+         (root (options-root options)))
+    (cute display (verify-pipeline "aut" root model))))
+
 (define (in-out:mcrl2->lps options)
   (let ((debug? (dzn:command-line:get 'debug))
         (timings? (dzn:command-line:get 'timings)))
@@ -250,10 +272,17 @@ actions."
 
 (define (in-out:aut->aut-weak-trace options)
   (let* ((model (options-model options))
-         (model-name (makreel:name model))
-         (taus (if (is-a? model <interface>)
-                   '("--tau=inevitable,optional")
-                   '()))
+         (taus (if (not (is-a? model <interface>)) '()
+                   '("--tau=inevitable,optional,tag,<flush>")))
+         (timings? (dzn:command-line:get 'timings)))
+    `("ltsconvert" ,(if timings? "--timings" "--quiet")
+      "-eweak-trace" ,@taus "--in=aut" "--out=aut")))
+
+(define (in-out:lts-hide-internal-labels options)
+  (let* ((model (options-model options))
+         (name (makreel:full-name model))
+         (taus (if (not (is-a? model <interface>)) '()
+                   `(,(format #f "--tau=~aflush,~ainternal,tag" name name))))
          (timings? (dzn:command-line:get 'timings)))
     `("ltsconvert" ,(if timings? "--timings" "--quiet")
       "-eweak-trace" ,@taus "--in=aut" "--out=aut")))
@@ -280,17 +309,21 @@ actions."
          (exclude-taus (if (is-a? model <interface>) ""
                            (component-exclude-taus model))))
     (append
-      (if (string-null? taus) '()
-          (list (string-append "--tau=" taus)))
-      (if (string-null? exclude-taus) '()
-          (list (string-append "--exclude-tau=" exclude-taus))))))
+     (if (string-null? taus) '()
+         (list (string-append "--tau=" taus)))
+     (if (string-null? exclude-taus) '()
+         (list (string-append "--exclude-tau=" exclude-taus))))))
 
 (define (in-out:aut->verify-interface options)
-  (let ((taus (model-taus options)))
+  (let* ((taus (model-taus options))
+         (model (options-model options))
+         (tags (options-tags options))
+         (tags (string-join tags ";")))
     `(,%dzn "lts" "--single-line"
             "--deadlock"
             ,@taus
             "--livelock"
+            ,@(if (%no-unreachable?) '() '("--tags"))
             "-")))
 
 (define in-out:aut->verify-interface-nondet
@@ -300,6 +333,8 @@ actions."
 (define (in-out:aut->verify-component options)
   (let* ((taus (model-taus options))
          (model (options-model options))
+         (tags (options-tags options))
+         (tags (string-join tags ";"))
          (deterministic (deterministic-labels model)))
     `(,%dzn "lts" "--single-line"
             "--deterministic-labels" ,deterministic
@@ -307,17 +342,17 @@ actions."
             "--deadlock"
             ,@taus
             "--livelock"
+            ,@(if (%no-unreachable?) '() '("--tags"))
             "--failures"
             "-")))
 
 (define (in-out:aut+provides-aut->verify-compliance options)
   (let* ((model (options-model options))
-         (timings? (dzn:command-line:get 'timings))
          (taus (compliance-taus model))
          (taus (if (string-null? taus) '()
-                   (list (string-append "--tau=" taus)))))
-    `("ltscompare"
-      ,(if timings? "--timings" "--quiet")
+                   (list (string-append "--tau=" taus))))
+         (timings? (dzn:command-line:get 'timings)))
+    `("ltscompare" ,(if timings? "--timings" "--quiet")
       "--counter-example" "--structured-output" "-pweak-failures"
       ,@taus
       "--in1=aut" "--in2=aut" "-" "-")))
@@ -328,15 +363,17 @@ actions."
     (("lps"                     "lpsconstelm")             . ,in-out:lps->lpsconstelm)
     (("lpsconstelm"             "lpsparelm")               . ,in-out:lps->lpsparelm)
     (("lpsparelm"               "maut")                    . ,in-out:lps->aut)
-    (("maut"                    "aut")                     . ,in-out:maut->aut)
-    (("maut"                    "maut-weak-trace")         . ,in-out:aut->aut-weak-trace)
     (("maut"                    "maut-dpweak-bisim")       . ,in-out:aut->aut-dpweak-bisim)
-    (("maut-weak-trace"         "aut-weak-trace")          . ,in-out:maut->aut)
+    (("maut"                    "aut")                     . ,in-out:maut->aut)
     (("maut-dpweak-bisim"       "aut-dpweak-bisim")        . ,in-out:maut->aut)
+    (("maut-weak-trace"         "aut-weak-trace")          . ,in-out:maut->aut)
+    (("maut"                    "maut-weak-trace")         . ,in-out:aut->aut-weak-trace)
+    (("maut"                    "maut-weak-trace+hide")    . ,in-out:lts-hide-internal-labels)
     (("aut-dpweak-bisim"        "aut-failures")            . ,in-out:aut->aut-failures)
+    (("dzn"                     "aut-cached")              . ,in-out:dzn->aut-cached)
     (("dzn"                     "aut-dpweak-bisim-cached") . ,in-out:dzn->aut-dpweak-bisim-cached)
     (("aut-dpweak-bisim-cached" "verify-interface")        . ,in-out:aut->verify-interface)
-    (("aut-dpweak-bisim-cached" "aut-weak-trace-cached")   . ,in-out:aut->aut-weak-trace)
+    (("aut-cached"              "aut-weak-trace-cached")   . ,in-out:aut->aut-weak-trace)
     (("aut-weak-trace-cached"   "verify-interface-nondet") . ,in-out:aut->verify-interface-nondet)
     (("aut-dpweak-bisim"        "verify-component")        . ,in-out:aut->verify-component)
     (("dzn"                     "aut+provides-aut")        . ,in-out:dzn->aut+provides-aut)
@@ -370,6 +407,15 @@ for MODEL, using ROOT."
       (let ((imports (command-line:get 'import)))
         (if (null? imports) ""
             (string-join imports " -I " 'prefix))))
+    (define (flag->string flag)
+      (if (command-line:get flag) (format #f " --~a" flag)
+          ""))
+    (define (flags->string)
+      (string-append
+       (imports->string)
+       (if (%no-unreachable?) " --no-unreachable" "")
+       (flag->string 'no-constraint)
+       (flag->string 'no-non-compliance)))
     (let ((file-name (ast:source-file root))
           (model-name (makreel:unticked-dotted-name model)))
       (match command
@@ -378,14 +424,16 @@ for MODEL, using ROOT."
            (format #f "~a ~a" program (string-join (map arg->string args)))))
         ((? (const (equal? out "verify-compliance")))
          (format #f "~a verify --model=~a --out=aut+provides-aut~a ~a"
-                 (program->string %dzn) model-name (imports->string) file-name))
+                 (program->string %dzn) model-name (flags->string) file-name))
         (_
          (and (not (string-prefix? "verify-interface" out))
               (string-append
-               (format #f "~a code --language=makreel --model=~a~a --init=~s  ~a"
-                       (program->string %dzn) model-name (imports->string)
-                       (get-init model)
-                       file-name)))))))
+               (format
+                #f
+                "~a code --language=makreel --model=~a --init=~s~a -o- ~a"
+                (program->string %dzn) model-name (get-init model)
+                (flags->string)
+                file-name)))))))
   (string-join (filter-map command->string commands) " \\\n  | "))
 
 (define* (unmemoized-verify-pipeline out root model #:key (init (get-init model)) stdout?)
@@ -395,14 +443,15 @@ to (current-output-port)."
   (define ((prepare options) next result)
     (let ((next (if (procedure? next) (next options) next)))
       (cons next result)))
-  (let* ((options (make-options root model init))
+  (let* ((tags (model-tags model))
+         (options (make-options root model init tags))
          (commands (get-commands in-out.pipeline out))
          (commands (reverse (fold (prepare options) '() commands))))
     (when (dzn:command-line:get 'debug)
       (let ((commands (pretty-verify-pipeline commands out root model)))
-       (if (equal? out "aut-dpweak-bisim")
-           (format (current-error-port) "~a\\\n  | " commands)
-           (format (current-error-port) "~a\n" commands))))
+        (if (equal? out "aut-dpweak-bisim")
+            (format (current-error-port) "~a\\\n  | " commands)
+            (format (current-error-port) "~a\n" commands))))
     (let* ((pipeline (if stdout? pipeline->port pipeline->string))
            (result status (pipeline commands)))
       (values result status))))
@@ -416,7 +465,7 @@ to (current-output-port)."
       (list result status))))
 
 (define memoizing-verify-pipeline
-  (pure-funcq verify-pipeline-wrapper))
+  (perfect-funcq 1024 verify-pipeline-wrapper))
 
 (define* (verify-pipeline out root model #:key (init (get-init model)))
   "Create a verify pipeline to produce OUT from MODEL.  Use standard
@@ -443,7 +492,8 @@ init for MODEL unless INIT."
       ((assert (and (? (cute string-prefix? "fail:" <>)) fail))
        (cons assert (split-colon fail)))
       (_ line)))
-  (let ((result (map split-colon (string-split result #\newline))))
+  (let* ((lines (string-split result #\newline))
+         (result (map split-colon lines)))
     (map split-fail result)))
 
 (define (semi->newline string)
@@ -451,16 +501,23 @@ init for MODEL unless INIT."
 
 (define (get-line key result)
   (let ((key (symbol->string key)))
-    (or (find (compose (cute equal? key <>) car) result)
-        (throw 'programming-error (format #f "no such assert: ~s, result: ~s\n" key result)))))
+    (find (compose (cute equal? key <>) car) result)))
 
 (define (get-lts result)
   (let ((line (get-line 'failures result)))
-   (match line
-     (("failures" lts)
-      (semi->newline lts))
-     (_
-      (throw 'programming-error (format #f "no failures lts: ~s, line: ~s\n" line result))))))
+    (match line
+      (("failures" lts)
+       (semi->newline lts))
+      (_
+       (throw 'programming-error
+              (format #f "no failures lts: ~s, line: ~s\n" line result))))))
+
+(define (get-tags result)
+  (let ((line (get-line 'tags result)))
+    (match line
+      (("tags" lts)
+       (string-split lts #\;))
+      (_ #f))))
 
 (define (get-trace key result)
   (let ((assert (get-line key result)))
@@ -470,21 +527,43 @@ init for MODEL unless INIT."
       ((assert "fail" trace)
        (let ((trace (semi->newline trace)))
          (hide-internal-labels trace)))
-      (_
-       (throw 'programming-error (format #f "ill-formed assert: ~s, result: ~s\n" assert result))))))
+      (_ #f))))
 
-(define (report-ok model-type model-name assert)
-  (let ((verbose? (dzn:command-line:get 'verbose)))
+(define (assert-unreachable lts-tags model-tags)
+  (define (tag< a b)
+    (define (tag->list tag)
+      (map (compose string->number string-trim-both) (string-split tag #\,)))
+    (match (cons (tag->list a) (tag->list b))
+      (((line-a column-a) . (line-b column-b))
+       (or (< line-a line-b)
+           (and (= line-a line-b) (< column-a column-b))))))
+  (let ((missing (lset-difference equal? model-tags lts-tags)))
+    (if (null? missing)
+        (list "unreachable" "ok")
+        (list "unreachable" "fail" (string-join missing ";")))))
+
+(define (report-ok model assert)
+  (let ((verbose? (dzn:command-line:get 'verbose))
+        (model-name (makreel:unticked-dotted-name model)))
     (when (dzn:command-line:get 'verbose)
-      (format (current-error-port) "verify: ~a: check: ~a: ok\n" model-name assert))
+      (format (current-error-port)
+              "verify: ~a: check: ~a: ok\n" model-name assert))
     #f))
 
-(define (report-fail model-type model-name assert trace)
+(define (report-fail model assert trace)
   (define (remove-flushes trace)
     (filter (negate (cut string-contains <> "<flush>")) trace))
   (define (drop-queue-full-tail trace)
     (append (take-while (negate (cut equal? "<queue-full>" <>)) trace) (list "<queue-full>")))
-  (let* ((trace (filter (negate string-null?) (string-split trace #\newline)))
+  (define (tag->message tag)
+    (apply format #f "~a:~a:~a: error: code will never be executed"
+           (.file-name (.location (.behavior model)))
+           (map string-trim-both (string-split tag #\,))))
+  (let* ((model-name (makreel:unticked-dotted-name model))
+         (unreachable? (eq? assert 'unreachable))
+         (trace (filter (negate string-null?) (string-split trace #\newline)))
+         (tags (and unreachable? trace))
+         (trace (if unreachable? '() trace))
          (last-el (and (pair? trace) (last trace)))
          (second-last (and (pair? trace)
                            (pair? (drop-right trace 1))
@@ -497,8 +576,8 @@ init for MODEL unless INIT."
                                          <second-reply>
                                          <range-error>
                                          <type-error>))))
-                    (if keep-error? last
-                        assert)))
+                     (if keep-error? last
+                         assert)))
                   ((illegal)
                    (if (equal? last '<queue-full>) last
                        assert))
@@ -511,10 +590,13 @@ init for MODEL unless INIT."
          (message (case error
                     ((illegal) (format #f "illegal action performed in model ~a" model-name))
                     ((non-deterministic)
-                     (case model-type
-                       ((component) (format #f "component ~a is non-deterministic due to overlapping guards" model-name))
-                       ((interface) (format #f "interface ~a is unobservably non-deterministic" model-name))))
+                     (cond
+                      ((is-a? model <interface>)
+                       (format #f "interface ~a is unobservably non-deterministic" model-name))
+                      ((is-a? model <component>)
+                       (format #f "component ~a is non-deterministic" model-name))))
                     ((non-compliance) (format #f "component ~a is non-compliant with interface(s) of provides port(s)" model-name))
+                    ((unreachable) (string-join (map tag->message tags) "\n"))
                     ((<range-error>) (format #f "integer range error in model ~a" model-name))
                     ((<type-error>) (format #f "type error in model ~a" model-name))
                     ((<missing-reply>) (format #f "reply missing from model ~a" model-name))
@@ -529,21 +611,26 @@ init for MODEL unless INIT."
          (trace (string-join trace "\n")))
     (when (dzn:command-line:get 'verbose)
       (format (current-error-port) "verify: ~a: check: ~a: fail\n" model-name assert))
-    (format (current-error-port) "error: ~a\n" message)
+    (if unreachable? (write-line message (current-error-port))
+        (format (current-error-port) "error: ~a\n" message))
     (unless (string-null? trace)
       (format #t "model: ~a\n" model-name)
       (format #t "~a\n" trace))
-    #t))
+    (not unreachable?)))
 
-(define (report-skip model-type model-name assert)
-  (when (dzn:command-line:get 'verbose)
-    (format (current-error-port) "verify: ~a: check: ~a: skip\n" model-name assert))
-  #f)
+(define (report-skip model assert)
+  (let ((verbose? (dzn:command-line:get 'verbose))
+        (model-name (makreel:unticked-dotted-name model)))
+    (when verbose?
+      (format (current-error-port)
+              "verify: ~a: check: ~a: skip\n"
+              model-name assert))
+    #f))
 
-(define (report assert skip trace model-type model-name)
-  (cond (skip  (report-skip model-type model-name assert))
-        (trace (report-fail model-type model-name assert trace))
-        (else  (report-ok   model-type model-name assert))))
+(define (report assert skip? trace model)
+  (cond (skip? (report-skip model assert))
+        (trace (report-fail model assert trace))
+        (else  (report-ok   model assert))))
 
 
 ;;;
@@ -555,17 +642,27 @@ init for MODEL unless INIT."
       (fold (lambda (e res) (or res (e))) #f l)))
 
 (define (mcrl2:verify-interface-asserts model root)
-  (let* ((model-name (makreel:unticked-dotted-name model))
-         (result (string-append
-                  (verify-pipeline "verify-interface" root model)
-                  (verify-pipeline "verify-interface-nondet" root model)))
-         (result (result-split result)))
-    (define (report-assert assert)
-      (report assert #f (get-trace assert result) 'interface model-name))
-    (reduce-or (command-line:get 'all)
-               (list (cute report-assert 'deadlock)
-                     (cute report-assert 'livelock)
-                     (cute report-assert 'deterministic)))))
+  (and
+   (not (command-line:get 'no-interfaces))
+   (let* ((model-name (makreel:unticked-dotted-name model))
+          (asserts (verify-pipeline "verify-interface" root model))
+          (asserts (result-split asserts))
+          (nondets (verify-pipeline "verify-interface-nondet" root model))
+          (nondets (result-split nondets))
+          (lts-tags (get-tags asserts))
+          (unreachable (assert-unreachable lts-tags (model-tags model)))
+          (result `(,unreachable
+                    ,@asserts
+                    ,@nondets))
+          (deadlock? (get-trace 'deadlock result)))
+     (define* (report-assert assert #:key skip?)
+       (report assert skip? (get-trace assert result) model))
+     (reduce-or (command-line:get 'all)
+                `(,(cute report-assert 'deadlock)
+                  ,@(if (%no-unreachable?) '()
+                        `(,(cut report-assert 'unreachable #:skip? deadlock?)))
+                  ,(cute report-assert 'livelock)
+                  ,(cute report-assert 'deterministic))))))
 
 (define (mcrl2:verify-compliance root model)
   (let* ((output status (verify-pipeline "verify-compliance" root model))
@@ -609,31 +706,38 @@ init for MODEL unless INIT."
 
 (define (mcrl2:verify-component-asserts model root)
   (let* ((model-name (makreel:unticked-dotted-name model))
-         (result status (verify-pipeline "verify-component" root model))
+         (result status (verify-pipeline (component-stage) root model))
          (result (result-split result))
+         (lts-tags (get-tags result))
+         (unreachable (assert-unreachable lts-tags (model-tags model)))
+         (result `(,unreachable
+                   ,@result))
+         (illegal? (get-trace 'illegal result))
+         (deadlock? (get-trace 'deadlock result))
          (refinement-trace interface-accepts component-accepts
                            (mcrl2:verify-compliance root model)))
-    (define (report-assert assert)
-      (report assert #f (get-trace assert result) 'component model-name))
+    (define* (report-assert assert #:key skip? trace)
+      (report assert skip? (or trace (get-trace assert result)) model))
     (define (extend-trace trace accepts)
       (if accepts (string-append trace (car accepts) "\n")
           trace))
     (reduce-or (command-line:get 'all)
-               (list (cut report-assert 'deterministic)
-                     (cut report-assert 'illegal)
-                     (cut report-assert 'deadlock)
-                     (cut report-assert 'livelock)
-                     (cut report 'compliance
-                          (or (get-trace 'illegal result) (get-trace 'deadlock result))
-                          refinement-trace
-                          'component model-name)))))
+               `(,(cute report-assert 'deterministic)
+                 ,(cute report-assert 'illegal)
+                 ,(cute report-assert 'deadlock)
+                 ,@(if (%no-unreachable?) '()
+                       `(,(cut report-assert 'unreachable
+                               #:skip? (or illegal? deadlock?))))
+                 ,(cute report-assert 'livelock)
+                 ,(cut report-assert 'compliance
+                       #:skip? (or illegal? deadlock?)
+                       #:trace refinement-trace)))))
 
-(define (mcrl2:verify-interface root model-name)
-  (let ((model (makreel:get-model root model-name)))
-    (mcrl2:verify-interface-asserts model root)))
+(define (mcrl2:verify-interface root model)
+  (mcrl2:verify-interface-asserts model root))
 
-(define (mcrl2:verify-component root model-name)
-  (let* ((component (makreel:get-model root model-name))
+(define (mcrl2:verify-component root model)
+  (let* ((component model)
          (interfaces (delete-duplicates (map .type (ast:port* component)) ast:eq?))
          (verify-models (append (map (lambda (i) (cut mcrl2:verify-interface-asserts i root)) interfaces)
                                 (list (cut mcrl2:verify-component-asserts component root)))))
@@ -641,8 +745,8 @@ init for MODEL unless INIT."
 
 (define (mcrl2:verify root model-name)
   (let ((model (makreel:get-model root model-name)))
-    (cond ((is-a? model <interface>) (mcrl2:verify-interface root model-name))
-          ((is-a? model <component>) (mcrl2:verify-component root model-name))
+    (cond ((is-a? model <interface>) (mcrl2:verify-interface root model))
+          ((is-a? model <component>) (mcrl2:verify-component root model))
           (else #f))))
 
 
@@ -656,10 +760,10 @@ init for MODEL unless INIT."
 
 (define* (verification:verify options root #:key all? model-name)
   (define (model-names-for-verification root)
-    (let* ((models (ast:model* root))
+    (let* ((models (ast:model** root))
            (components (filter (conjoin (is? <component>) (negate ast:imported?) .behavior) models))
            (component-names (map makreel:unticked-dotted-name components))
-           (interfaces (filter (conjoin (is? <interface>) (negate ast:dzn-scope?)) models))
+           (interfaces (filter (is? <interface>) models))
            (interface-names (map makreel:unticked-dotted-name interfaces))
            (interface-names (let loop ((components components) (interface-names interface-names))
                               (if (null? components) interface-names
