@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2020, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -38,7 +38,9 @@
 
   #:export (%compliance-check?
             check-provides-compliance
-            check-provides-compliance*))
+            check-provides-compliance*
+            internal-check-compliance
+            update-shared-state))
 
 ;;; Commentary:
 ;;;
@@ -117,6 +119,7 @@ format."
                               trace-suffix))))
          (status (.status (car trace)))
          (error? (and (is-a? status <error>)
+                      (not (is-a? status <queue-full-error>))
                       (not (is-a? status <illegal-error>))
                       (not (is-a? status <implicit-illegal-error>))))
          (trace (if merged? trace
@@ -233,19 +236,80 @@ provides port, mark the trace as <fork-error>, otherwise return false."
                                              #:message "non-compliance"))))
                   (list (cons pc trace))))))))
 
-(define (check-provides-compliance pc event trace)
-  "Check TRACE for traces-compliance with the provides ports, for EVENT.
-Update the state of the provides port in TRACE for EVENT.  For a blocked
+(define (run-provides-modeling pc port-instance event)
+  (%debug "run-provides-modeling... ~a\n" event)
+  (let* ((ipc (clone pc #:instance port-instance #:statement #f))
+         (interface ((compose .type .ast) port-instance))
+         (modeling-names (modeling-names interface)))
+    (parameterize ((%sut port-instance)
+                   (%exploring? #t)
+                   (%strict? #f))
+      (append-map (cute run-to-completion ipc <>) modeling-names))))
+
+(define (run-provides-port pc port-instance event)
+  (%debug "run-provides-port... ~a ~a\n" port-instance event)
+  (let* ((ipc (clone pc #:previous #f #:trail '() #:status #f #:statement #f))
+         (ipc (reset-reply ipc port-instance)))
+    (%debug "  ipc: ~a\n" ipc)
+    (parameterize ((%sut port-instance)
+                   (%exploring? #t))
+      (run-to-completion ipc event))))
+
+(define (port-acceptance-equal? a b)
+  (and (equal? (and=> (caar a) trigger->string)
+               (and=> (caar b) trigger->string))
+       (equal? (cadr a) (cadr b))))
+
+(define* (compliance-first-non-match sut-trail port-instance port-trace
+                                     #:key blocked? provides-trigger?)
+  (define (non-matching-pair? a b)
+    (and (not (equal? (cdr a)
+                      ((compose last (cut string-split <> #\.) cdr) b)))
+         (cons a b)))
+
+  (let* ((port-trail (parameterize ((%sut port-instance))
+                       (trace->trail port-trace)))
+         (port-length (length port-trail))
+         (sut-length (length sut-trail))
+         (port-next (and (> port-length sut-length)
+                         (and=> (list-ref port-trail sut-length) cdr)))
+         (truncate? (and provides-trigger?
+                         blocked?
+                         (not (equal? port-next "<illegal>"))))
+         (port-trail (if (not truncate?) port-trail
+                         ;; Check prefix only as long as trace is blocked
+                         (list-head port-trail (min (length port-trail)
+                                                    (length sut-trail)))))
+         (foo (%debug "     port trail : ~s\n" (map cdr port-trail)))
+         (port-name ((compose .name .ast) port-instance))
+         (events (map (compose last (cut string-split <> #\.) cdr) sut-trail))
+         (foo (%debug "      sut trail : ~s\n\n" events)))
+    (or (any non-matching-pair? port-trail sut-trail)
+        (let ((port-length (length port-trail))
+              (sut-length (length sut-trail)))
+          (cond ((< port-length sut-length) (cons '(#f) (list-ref sut-trail port-length)))
+                ((> port-length sut-length) (list (list-ref port-trail sut-length) #f))
+                (else #f))))))
+
+(define-method (check-provides-compliance (pc <program-counter>)
+                                          (instance <runtime:instance>)
+                                          trigger trace)
+  "Check TRACE for traces-compliance with the provides ports of INSTANCE, for EVENT.
+Update the state of the provides port in TRACE for TRIGGER.  For a blocked
 trace, the check is done in an incremental way: only the part that the
 component has executed is considered.
 
 Return a list of traces, possibly marked with <compliance-error>."
 
   (define (drop-prefix pc trigger trace)
-    (let* ((r:port (runtime:port-name->instance (.port.name trigger)))
-           (r:component-port (runtime:other-port r:port))
+    (let* ((port (.port trigger))
+           (r:port (runtime:port-name->instance (.name port)))
+           (r:component-port (if r:port (runtime:other-port r:port)
+                                 (runtime:port instance port)))
+           (component-trigger
+            (if r:port (trigger->component-trigger trigger)
+                (trigger->component-trigger r:component-port trigger)))
            (instance (.container r:component-port))
-           (component-trigger (trigger->component-trigger trigger))
            (at (list-index
                 (if (ast:provides? component-trigger)
                     (conjoin
@@ -264,60 +328,45 @@ Return a list of traces, possibly marked with <compliance-error>."
                       (filter (compose (negate (is? <trigger-return>)) .statement) trace))))
       trace))
 
-  (let* ((component ((compose .type .ast) (%sut)))
-         (trigger (and (string? event)
-                       (not (equal? event "<defer>"))
-                       (clone (string->trigger event) #:parent component)))
+  (let* ((event (and=> trigger trigger->string))
          (blocking? (find (compose pair? .blocked) trace))
-         (sut-trace (if (or (not trigger) (not blocking?)) trace
+         (foo? (or (not trigger) (not (.port trigger)) (not blocking?)))
+         (drop? (and trigger (.port trigger) blocking?))
+         (sut-trace (if (not drop?) trace
                         (drop-prefix pc trigger trace)))
-         (sut-trail (trace->trail sut-trace))
-         (provides-event (any (compose (conjoin (disjoin (is? <action>)
-                                                         (is? <trigger>))
-                                                ast:provides?
-                                                identity)
-                                       .statement)
+         (internal? (and (is-a? (%sut) <runtime:system>)
+                         (not (eq? instance (%sut)))))
+         (sut-trail (if (not internal?) (trace->trail sut-trace)
+                        (trace->component-trail sut-trace)))
+         (component (runtime:ast-model instance))
+         (provides-event (any (conjoin
+                               (compose (cute eq? <> instance) .instance)
+                               (compose (is? <initial-compound>) .statement)
+                               .trigger)
                               (reverse trace)))
-         (provides-event (and=> provides-event .event.name))
-         (provides-trigger? (or (provides-trigger? provides-event)
+         (event-name (and event (.event.name (string->trigger event))))
+         (provides-event (and provides-event
+                              (equal? (.event.name provides-event)
+                                      event-name)
+                              provides-event))
+         (provides-trigger? (or (and=> (as provides-event <trigger>) ast:provides?)
                                 (provides-trigger? event)))
          (port-event (and provides-trigger? (.event.name trigger)))
          (port (and provides-trigger? (.port trigger))))
 
     (define (check-compliance port traces)
       (let* ((trace (if (null? traces) '() (car traces)))
-             (port-name (.name port))
-             (port-instance (runtime:port-name->instance port-name)))
-
-        (define (port-event? e)
-          (and (string? e)
-               (match (string-split e #\.)
-                 (('state state) #f)
-                 ((port event) (and (equal? port port-name) event))
-                 (_ #f))))
-
-        (define (run-provides-modeling ipc port-instance)
-          (%debug "run-provides-modeling... ~a\n" event)
-          (let* ((ipc (clone pc #:instance port-instance #:statement #f))
-                 (interface ((compose .type .ast) port-instance))
-                 (modeling-names (modeling-names interface)))
-            (parameterize ((%sut port-instance)
-                           (%exploring? #t)
-                           (%strict? #f))
-              (append-map (cute run-to-completion ipc <>) modeling-names))))
-
-        (define (run-provides-port pc event)
-          (%debug "run-provides-port... ~a ~a\n" port-instance event)
-          (%debug "  pc: ~a\n" pc)
-          (parameterize ((%sut port-instance)
-                         (%exploring? #t))
-            (run-to-completion pc event)))
+             (r:port (runtime:port instance port))
+             (port-name (if (not internal?) (.name port)
+                            (trace-name r:port)))
+             (port-instance
+              (if (not internal?) (runtime:port-name->instance port-name)
+                  (runtime:port instance port))))
 
         (%debug "check-provides-compliance... ~s: ~a [~a]\n" port-name event port-event)
-        (let* ((ipc (clone pc #:previous #f #:trail '() #:status #f #:statement #f))
-               (ipc (reset-reply ipc port-instance))
-               (port-traces (if port-event (run-provides-port ipc port-event)
-                                (run-provides-modeling ipc port-instance)))
+        (let* ((port-traces
+                (if port-event (run-provides-port pc port-instance port-event)
+                    (run-provides-modeling pc port-instance event)))
                (port-prefix (format #f "~a." port-name))
                (sut-trail (filter (compose (disjoin
                                             (cute equal? <> "<illegal>")
@@ -328,49 +377,20 @@ Return a list of traces, possibly marked with <compliance-error>."
                               (or (pair? (.blocked (car trace)))
                                   (find blocked-on-boundary? trace)))))
 
-          (define (port-trace->trail trace)
-            (parameterize ((%sut port-instance)) (trace->trail trace)))
-
           (define (first-non-match port-trace)
-            (define (non-matching-pair? a b)
-              (and (not (equal? (cdr a) ((compose last (cut string-split <> #\.) cdr) b))) (cons a b)))
-
-            (let* ((port-trail (port-trace->trail port-trace))
-                   (port-length (length port-trail))
-                   (sut-length (length sut-trail))
-                   (port-next (and (> port-length sut-length)
-                                   (and=> (list-ref port-trail sut-length) cdr)))
-                   (truncate? (and provides-trigger?
-                                   blocked?
-                                   (not (equal? port-next "<illegal>"))))
-                   (port-trail (if (not truncate?) port-trail
-                                   ;; Check prefix only as long as trace is blocked
-                                   (list-head port-trail (min (length port-trail)
-                                                              (length sut-trail)))))
-                   (foo (%debug "     port trail : ~s\n" (map cdr port-trail)))
-                   (port-name ((compose .name .ast) port-instance))
-                   (events (map (compose last (cut string-split <> #\.) cdr) sut-trail))
-                   (foo (%debug "      sut trail : ~s\n\n" events)))
-              (or (any non-matching-pair? port-trail sut-trail)
-                  (let ((port-length (length port-trail))
-                        (sut-length (length sut-trail)))
-                    (cond ((< port-length sut-length) (cons '(#f) (list-ref sut-trail port-length)))
-                          ((> port-length sut-length) (list (list-ref port-trail sut-length) #f))
-                          (else #f))))))
-
-          (define (port-acceptance-equal? a b)
-            (and (equal? (and=> (caar a) trigger->string)
-                         (and=> (caar b) trigger->string))
-                 (equal? (cadr a) (cadr b))))
+            (compliance-first-non-match sut-trail port-instance port-trace
+                                        #:blocked? blocked?
+                                        #:provides-trigger? provides-trigger?))
 
           (define (event-on-trail? event-name trace)
-            (let* ((trail (port-trace->trail trace))
+            (let* ((trail (parameterize ((%sut port-instance))
+                            (trace->trail trace)))
                    (trail (map cdr trail)))
               (member event-name trail)))
 
           (when (> (dzn:debugity) 0)
-            (%debug "sut-trail:~s\n" (map cdr sut-trail))
-            (%debug "port-traces[~s]:\n" (length port-traces))
+            (%debug "[c] sut-trail:~s\n" (map cdr sut-trail))
+            (%debug "[c] port-traces[~s]:\n" (length port-traces))
             (parameterize ((%sut port-instance))
               (display-trails port-traces)))
 
@@ -399,11 +419,14 @@ Return a list of traces, possibly marked with <compliance-error>."
              ((and (pair? port-traces)
                    (pair? trace))
               (%debug "  exit 2\n")
-              (let* ((port-pcs (map (compose (cut clone pc #:state <>) .state car) port-traces))
-                     (traces (map (lambda (port-pc)
-                                    (cons (set-state (car trace) (get-state port-pc port-instance))
-                                          (cdr trace)))
-                                  port-pcs)))
+              (let* ((port-pcs (map
+                                (compose (cut clone pc #:state <>) .state car)
+                                port-traces))
+                     (pc (car trace))
+                     (pcs (map (cute update-state pc port-instance <>)
+                               port-pcs))
+                     (traces (map (lambda (pc trace) (cons pc (cdr trace)))
+                                  pcs traces)))
                 (map (cute zip trigger <> <>) traces port-traces)))
              ((and (%compliance-check?)
                    (null? non-compliances)
@@ -474,10 +497,11 @@ Return a list of traces, possibly marked with <compliance-error>."
              (else
               (%debug "  exit 8\n")
               (let* ((port-trace (car non-compliances))
-                     (port-state (get-state (last port-trace)))
+                     (port-pc (last port-trace))
+                     (port-instance (.instance port-pc))
                      (port-trace
                       (rewrite-trace-head
-                       (cut set-state <> port-state)
+                       (cut update-state <> port-instance port-pc)
                        port-trace))
                      (trace (zip trigger trace (car non-compliances)))
                      (trace
@@ -509,6 +533,113 @@ Return a list of traces, possibly marked with <compliance-error>."
                        (check-requires-provides-fork trace)))
               (fold check-compliance (list trace) ports))))))
 
+(define-method (update-shared-state (pc <program-counter>)
+                                    (instance <runtime:instance>)
+                                    port trigger trace)
+  "Return PC with updated provides port state after last out-event found
+on TRACE."
+
+  (let* ((event (and=> trigger trigger->string))
+         (orig-pc pc)
+         (pc (reset-replies pc))
+         (pc (clone pc #:instance #f))
+         (internal? (and (is-a? (%sut) <runtime:system>)
+                         (not (eq? instance (%sut)))))
+         (sut-trail (if (not internal?) (trace->trail trace)
+                        (trace->component-trail trace)))
+         (component (runtime:ast-model instance))
+         (provides-event (any (conjoin
+                               (compose (cute eq? <> instance) .instance)
+                               (compose (is? <initial-compound>) .statement)
+                               .trigger)
+                              (reverse trace)))
+         (event-name (and event (.event.name (string->trigger event))))
+         (provides-event (and provides-event
+                              (equal? (.event.name provides-event)
+                                      event-name)
+                              provides-event))
+         (provides-trigger? (or (and=> (as provides-event <trigger>) ast:provides?)
+                                (provides-trigger? event)))
+         (port-event (and provides-trigger? (.event.name trigger))))
+
+    (let* ((r:port (runtime:port instance port))
+           (port-name (if (not internal?) (.name port)
+                          (trace-name r:port)))
+           (port-instance
+            (if (not internal?) (runtime:port-name->instance port-name)
+                (runtime:port instance port))))
+
+      (define (truncate actions trace)
+        (let* ((trace (reverse trace))
+               (trace
+                (let loop ((trace trace) (actions actions))
+                  (if (zero? actions) '()
+                      (match trace
+                        (() '())
+                        (((and pc (= .statement (? (is? <action>)))) rest ...)
+                         (cons pc (loop rest (1- actions))))
+                        ((pc rest ...)
+                         (cons pc (loop rest actions)))))))
+               (actions (filter (compose (is? <action>) .statement)
+                                trace)))
+          (reverse trace)))
+
+      (%debug "update-port-state... ~s: ~a [~a]\n" port-name event port-event)
+      (let* ((port-traces
+              (if port-event (run-provides-port pc port-instance port-event)
+                  (run-provides-modeling pc port-instance event)))
+             (port-prefix (format #f "~a." port-name))
+             (sut-trail (filter (compose (disjoin
+                                          (cute equal? <> "<illegal>")
+                                          (cute string-prefix? port-prefix <>))
+                                         cdr)
+                                sut-trail))
+             (actions (filter (compose (conjoin (is? <action>)
+                                                (compose (cute ast:eq? <> port)
+                                                         .port))
+                                       .statement)
+                              trace))
+             (actions (length actions))
+             (port-traces (map (cute truncate actions <>) port-traces))
+             (blocked? (and (pair? trace)
+                            (or (pair? (.blocked (car trace)))
+                                (find blocked-on-boundary? trace)))))
+
+        (define (first-non-match port-trace)
+          (compliance-first-non-match sut-trail port-instance port-trace
+                                      #:blocked? blocked?
+                                      #:provides-trigger? provides-trigger?))
+
+        (when (> (dzn:debugity) 0)
+          (%debug "[u] sut-trail:~s\n" (map cdr sut-trail))
+          (%debug "[u] port-traces[~s]:\n" (length port-traces))
+          (parameterize ((%sut port-instance))
+            (display-trails port-traces)))
+
+        (let ((port-traces
+               non-compliances
+               (partition (negate first-non-match) port-traces)))
+          (cond
+           ((pair? port-traces)
+            (%debug "  [u] exit 0\n")
+            (let* ((port-trace (car port-traces))
+                   (port-pc (car port-trace))
+                   (port-pc (clone pc #:state (.state port-pc)))
+                   (pc (car trace))
+                   (pc (update-state pc port-instance port-pc))
+                   (pc (clone pc #:instance instance)))
+              pc))
+           (else
+            (%debug "  [u] exit 1\n")
+            orig-pc)))))))
+
+(define-method (check-provides-compliance (pc <program-counter>) event trace)
+  (let* ((component ((compose .type .ast) (%sut)))
+         (trigger (and (string? event)
+                       (not (equal? event "<defer>"))
+                       (clone (string->trigger event) #:parent component))))
+    (check-provides-compliance pc (%sut) trigger trace)))
+
 (define* (check-provides-compliance+ pc event trace)
   "Run check-provides-compliance.  For a blocking trace that has been
 released by a requires event, also rerun check-provides-compliance for
@@ -581,3 +712,48 @@ TRACES."
     (append-map
      (cute check-provides-compliance+ pc event <>)
      traces))))
+
+(define-method (internal-check-compliance (pc <program-counter>) trace traces)
+  "Check component TRACE for compliance using PC as starting point,
+update TRACES."
+  (let* ((statement (.statement pc))
+         (trigger (if (is-a? statement <trigger-return>) (.trigger pc)
+                      (.trigger statement)))
+         (port (.port trigger))
+         (instance (.instance pc))
+         (r:port (runtime:port instance port))
+         (r:other-port (runtime:other-port r:port))
+         (trigger (if (not (is-a? trigger <q-trigger>)) trigger
+                      (trigger->component-trigger r:other-port trigger)))
+         (r:port
+          r:other-port
+          (if (is-a? statement <trigger-return>) (values r:port r:other-port)
+              (values r:other-port r:port)))
+         (instance (if (is-a? statement <trigger-return>) instance
+                       (.container r:port)))
+         (start-index (list-index
+                       (conjoin
+                        (compose (cute eq? <> instance) .instance)
+                        (compose (cute is-a? <> <initial-compound>) .statement))
+                       trace))
+         (internal-compliance? (and start-index
+                                    (not (runtime:boundary-port? r:other-port))
+                                    (null? (.blocked pc)))))
+
+    (%debug "internal-check-compliance... ~s: ~a [~a]\n"
+            (trace-name r:port) (trigger->string trigger) internal-compliance?)
+    (if (not internal-compliance?) traces
+        (let* ((sut-trace (list-head trace (1+ start-index)))
+               (cpc (reset-replies pc))
+               (cpc (clone cpc #:instance #f))
+               (event (trigger->string trigger))
+               (compliance-traces
+                (append-map
+                 (cute check-provides-compliance cpc instance trigger <>)
+                 (list sut-trace)))
+               (traces (map (cute
+                             rewrite-trace-head
+                             (cute update-state <> r:port <>) <> <>)
+                            traces
+                            (map car compliance-traces))))
+          traces))))
