@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2020, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
@@ -20,28 +20,23 @@
 ;;; License along with Dezyne.  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (dzn vm run)
-  #:use-module (ice-9 hcons)
-  #:use-module (ice-9 match)
-
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
   #:use-module (srfi srfi-71)
 
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
-  #:use-module (dzn goops)
+  #:use-module (ice-9 hcons)
+  #:use-module (ice-9 match)
 
   #:use-module (dzn ast)
-  #:use-module (dzn display)
+  #:use-module (dzn ast goops)
   #:use-module (dzn misc)
-  #:use-module (dzn parse)
-  #:use-module (dzn vm ast)
+  #:use-module (dzn vm compliance)
   #:use-module (dzn vm goops)
-  #:use-module (dzn vm runtime)
   #:use-module (dzn vm report)
+  #:use-module (dzn vm runtime)
   #:use-module (dzn vm step)
   #:use-module (dzn vm util)
   #:export (%exploring?
-            did-provides-out?
             extend-trace
             filter-compliance-error
             filter-error
@@ -49,20 +44,17 @@
             filter-implicit-illegal
             filter-implicit-illegal-only
             filter-match-error
-            flush-async
-            flush-async-trace
             flush-defer
             interactive?
             livelock?
             mark-livelock-error
-            run-async
-            run-async-event
             run-defer-event
             run-flush
             run-external
             run-external-q
             run-external-modeling
             run-requires
+            run-requires-flush
             run-silent
             run-to-completion
             run-to-completion*
@@ -157,7 +149,18 @@
   "Return #t when TRACES are a nondeterministic set, i.e.: at least two
 valid PCs that are executing an imperative statement."
   (let* ((pcs (map car traces))
-         (pcs (filter (negate .status) pcs)))
+         (pcs (filter (negate (is-status? <error>)) pcs))
+         (pcs (filter (disjoin (negate (is-status? <postponed-match>))
+                               (compose (conjoin
+                                         (negate (is? <action>))
+                                         (negate (is? <reply>))
+                                         (disjoin (negate (is? <variable>))
+                                                  (compose null?
+                                                           (cute tree-collect
+                                                                 (is? <action>)
+                                                                 <>))))
+                                        .statement))
+                      pcs)))
     (and
      (every (compose (cute is-a? <> <runtime:component>) .instance) pcs)
      (let ((declarative imperative
@@ -217,7 +220,7 @@ with a TRACE that has BLOCKED still set to this port."
      (let ((blocked-port (and (pair? blocked) (caar blocked)))
            (blocked-pc (and (pair? blocked) (cdar blocked))))
        (and
-        (not (ast:eq? statement (and=>  blocked-pc .statement)))
+        (not (ast:eq? statement (and=> blocked-pc .statement)))
         (or (and (is-a? (%sut) <runtime:system>)
                  (> (length (ast:provides-port* (runtime:%sut-model))) 1))
             (requires-modeling-armed? pc))
@@ -311,18 +314,18 @@ program-counters produced by taking a step."
               (and (is-a? statement  <action>)
                    (and=> (trace->trail orig-pc) cdr))
               input)
-     (cond ((.status pc)
-            pc)
-           ((or (and (%strict?) input)
-                (and input
-                     (is-a? statement <trigger-return>)
-                     (blocked-on-boundary? orig-pc)))
-            (let ((error (make <match-error> #:ast statement #:input input
-                               #:message "match")))
-              (clone pc #:status error)))
-           (else
-            (clone pc #:status (make <postponed-match> #:ast statement
-                                     #:input input))))))
+      (cond ((.status pc)
+             pc)
+            ((or (and (%strict?) input)
+                 (and input
+                      (is-a? statement <trigger-return>)
+                      (blocked-on-boundary? orig-pc)))
+             (let ((error (make <match-error> #:ast statement #:input input
+                                #:message "match")))
+               (clone pc #:status error)))
+            (else
+             (clone pc #:status (make <postponed-match> #:ast statement
+                                      #:input input))))))
 
   (define (blocked-collaterally? orig-pc pc)
     (match (.collateral pc)
@@ -343,6 +346,16 @@ program-counters produced by taking a step."
              => list)
             (else
              (let* ((o (.statement pc))
+                    (instance (.instance pc))
+                    (update-state?
+                     (and (is-a? instance <runtime:component>)
+                          (is-a? o <action>)
+                          (ast:out? o)))
+                    (pc (if (not update-state?) pc
+                            (let ((trigger (instance-rtc-trigger pc))
+                                  (port (.port o)))
+                              (update-shared-state pc instance port trigger
+                                                   trace))))
                     (observable? (or (is-a? o <action>)
                                      (is-a? o <q-out>)
                                      (is-a? o <trigger-return>)))
@@ -366,7 +379,32 @@ program-counters produced by taking a step."
                                 (map (cute clone <> #:trail (.trail pc)) pcs))
                                (else
                                 (map (mark-pc input pc) pcs))))
-                    (traces (map (cut cons <> trace) pcs)))
+                    (internal-compliance?
+                     (and (is-a? (%sut) <runtime:system>)
+                          (not (is-a? (%sut) <runtime:port>))
+                          (or (and
+                               (is-a? o <trigger-return>)
+                               (and=> (.port o) ast:provides?)
+                               (and=> (.trigger pc) .port)
+                               (ast:provides?
+                                (runtime:port instance (.port o))))
+                              (and (is-a? o <flush-return>)
+                                   (.trigger o)))))
+                    (traces (map (cut cons <> trace) pcs))
+                    (traces (if (not internal-compliance?) traces
+                                (internal-check-compliance pc trace traces)))
+                    (update? (and
+                              (not internal-compliance?)
+                              (is-a? (%sut) <runtime:system>)
+                              (is-a? o <trigger-return>)
+                              (runtime:boundary-port? instance)))
+                    (traces (if (not update?) traces
+                                (map
+                                 (cute rewrite-trace-head
+                                       (cut update-other-state <> instance
+                                            #:direction? ast:requires?)
+                                       <>)
+                                 traces))))
                (cond
                 ((ast:declarative? o)
                  (let ((declarative
@@ -411,7 +449,7 @@ PC until RTC?."
              (observe-pc (cadr trace))
              (trail (.trail pc))
              (input (.input (.status pc)))
-             (drop-event? (equal? input (observable observe-pc)))
+             (drop-event? (and (pair? trail) (equal? input (observable observe-pc))))
              (trail (if drop-event? (cdr trail) trail))
              (trace (rewrite-trace-head (cut clone <> #:status #f #:trail trail) trace)))
         trace))
@@ -444,6 +482,26 @@ PC until RTC?."
            (traces (filter (compose (cute equal? input <>) observable car) traces)))
       (loop (append-map extend-trace traces))))
 
+  (define (interface-non-deterministic? traces)
+    (let ((traces (filter (compose (is? <action>) .ast .status car) traces)))
+      (match traces
+        (((pc0 tail0 ...) (pcs tails ...) ...)
+         (define (state-equal? pc0 pc1)
+           (let ((variables0 (get-variables pc0 (%sut)))
+                 (variables1 (get-variables pc1(%sut))))
+             (ast:equal? variables0 variables1)))
+         (find (negate (cute state-equal? pc <>)) pcs))
+        (() #f))))
+
+  (define (interface-mark-determinism-error trace)
+    (match trace
+      ((pc-postponed pc trace ...)
+       (let* ((error (make <determinism-error>
+                       #:ast (.statement pc)
+                       #:message "non-deterministic"))
+              (pc (clone pc #:status error)))
+         (cons pc trace)))))
+
   (define (mark-end-of-trail traces)
     (define (set-end-of-trail labels pc)
       (let* ((statement (.statement pc))
@@ -489,9 +547,17 @@ PC until RTC?."
         =>
         (lambda (traces)
           (cond
+           ((and (not (%exploring?))
+                 (non-deterministic? traces))
+            (let ((traces (filter-implicit-illegal traces)))
+              (map mark-determinism-error traces)))
            ((or (= (length traces) 1)
                 (= (length (choice-labels traces)) 1))
             (reset-posponed-match traces))
+           ((and (is-a? (%sut) <runtime:port>)
+                 (%strict?)
+                 (interface-non-deterministic? traces))
+            (map interface-mark-determinism-error traces))
            ((and (not (%exploring?)) (interactive?))
             (choose-postponed-match traces))
            (else
@@ -525,7 +591,7 @@ with EVENT as first step, until RTC?."
                                      ((eq? event 'rtc) "*rtc*")
                                      (else (trigger->string event))))
                  (sut (%sut))
-                 (root (parent (.ast sut) <root>))
+                 (root (ast:parent (.ast sut) <root>))
                  (key (string-append
                        (number->string (.id root))
                        (parameterize ((%sut #f)) (runtime:dotted-name sut))
@@ -635,11 +701,22 @@ until RTC?."
     (append-map (cute run-external-modeling pc <>) ports)))
 
 (define-method (run-external-modeling (pc <program-counter>) event)
+  (define (event-executed? port-instance trace)
+    (let ((status (.status (car trace)))
+          (trail (map cdr (trace->trail trace))))
+      (or (is-a? status <queue-full-error>)
+          (let* ((pc (car trace))
+                 (pc trigger (dequeue-external pc port-instance)))
+            (and trigger
+                 (let ((trigger (trigger->component-trigger port-instance
+                                                            trigger)))
+                   (equal? (trigger->string trigger) event)))))))
   (let* ((component ((compose .type .ast) (%sut)))
          (trigger (clone (string->trigger event) #:parent component))
          (port-name (.port.name trigger))
          (port-instance (runtime:port-name->instance port-name))
-         (traces (run-external-modeling pc port-instance)))
+         (traces (run-external-modeling pc port-instance))
+         (traces (filter (cute event-executed? port-instance <>) traces)))
     (map car traces)))
 
 (define-method (run-interface (pc <program-counter>) (event <string>))
@@ -667,6 +744,27 @@ until RTC?."
                        (filter event-executed? traces))))
         traces)))))
 
+(define-method (run-requires-flush (pc <program-counter>) event)
+  (let* ((component ((compose .type .ast) (%sut)))
+         (trigger (string->trigger event))
+         (trigger (clone trigger #:parent component))
+         (port-name (.port.name trigger))
+         (port-instance (runtime:port-name->instance port-name))
+         (component-port (runtime:other-port port-instance))
+         (component-trigger (trigger->component-trigger component-port
+                                                        trigger))
+         (instance (.container component-port))
+         (collateral-blocked? (or (blocked-on-boundary? pc)
+                                  (and instance
+                                       (get-handling pc instance)
+                                       (blocked-port pc instance))))
+         (traces (if collateral-blocked? (list (list pc))
+                     (run-flush pc instance))))
+    traces))
+
+(define-method (run-requires-flush trace event)
+  (extend-trace trace (cute run-requires-flush <> event)))
+
 (define-method (run-requires (pc <program-counter>) event)
   (define (event-executed? port-instance trace)
     (let ((status (.status (car trace)))
@@ -681,7 +779,8 @@ until RTC?."
                       (equal? (trigger->string trigger) event)))))))
   (%debug "run-requires... ~s\n" event)
   (let* ((component ((compose .type .ast) (%sut)))
-         (trigger (clone (string->trigger event) #:parent component))
+         (trigger (string->trigger event))
+         (trigger (clone trigger #:parent component))
          (port-name (.port.name trigger))
          (port-instance (runtime:port-name->instance port-name))
          (interface ((compose .type .ast) port-instance))
@@ -691,75 +790,10 @@ until RTC?."
          (modeling-events (map (cute string-append port-name "." <>) modeling-names))
          (traces (append-map (cute run-to-completion pc <>) modeling-events))
          (traces (filter (cute event-executed? port-instance <>) traces))
+         (traces (filter-match-error traces))
          (errors (filter (compose .status car) traces)))
-
     (if (pair? errors) errors
-        (let* ((component-port (runtime:other-port port-instance))
-               (component-trigger (trigger->component-trigger component-port
-                                                              trigger))
-               (instance (.container component-port))
-               (collateral-blocked? (or (blocked-on-boundary? pc)
-                                        (and instance
-                                             (get-handling pc instance)
-                                             (blocked-port pc instance))))
-               (traces (if collateral-blocked? traces
-                           (append-map (cute run-flush <> instance) traces))))
-          traces))))
-
-(define-method (run-async-event (pc <program-counter>))
-  (let ((trace (step pc (make <flush-async>))))
-    (extend-trace trace run-to-completion-unmemoized)))
-
-(define-method (flush-async-trace (trace <list>) previous-trace)
-  (let ((trace (append trace previous-trace)))
-    (cond ((.status (car trace))
-           (list trace))
-          ((livelock? trace)
-           =>
-           (compose list (cute mark-livelock-error trace <>)))
-          (else
-           (let* ((pc (car trace))
-                  (traces (flush-async pc trace)))
-             (map (cute append <> trace) traces))))))
-
-(define (did-provides-out? trace)
-  (let* ((trail (map cdr (trace->trail trace)))
-         (r:ports (filter runtime:boundary-port? (%instances)))
-         (ports (map .ast r:ports))
-         (provide-ports (filter ast:provides? ports))
-         (provide-names (map .name provide-ports)))
-    (find (lambda (event)
-            (match (string-split event #\.)
-              ((port event) (member port provide-names))
-              (_ #f)))
-          trail)))
-
-(define-method (flush-async (pc <program-counter>) previous-trace)
-  (if (null? (.async pc)) '()
-      (let ((traces (run-async-event pc)))
-        (cond
-         ((null? traces) traces)
-         (else
-          (let* ((stop flush (partition
-                              (disjoin (compose null? .async car)
-                                       did-provides-out?
-                                       (compose .status car))
-                              traces))
-                 (traces (append-map (cute flush-async-trace <> previous-trace) flush))
-                 (livelock traces (partition livelock? traces))
-                 (livelock (map (cute mark-livelock-error <> <>)
-                                livelock
-                                (map livelock? livelock))))
-            (append stop
-                    livelock
-                    traces)))))))
-
-(define-method (flush-async (pc <program-counter>))
-  (flush-async pc '()))
-
-(define-method (flush-async-event (pc <program-counter>) event)
-  (let ((pc (clone pc #:trail (cons event (.trail pc)))))
-    (flush-async pc)))
+        (append-map (cute run-requires-flush <> event) traces))))
 
 (define-method (run-defer-event (pc <program-counter>) event)
   (%debug "run-defer-event ~a pc: ~s\n" event pc)
@@ -795,6 +829,12 @@ until RTC?."
                (traces (run-to-completion pc 'rtc))
                (blocked traces
                         (partition (compose blocked-on-boundary? car) traces))
+               (collateral-blocked? (or (blocked-on-boundary? pc)
+                                        (and instance
+                                             (get-handling pc instance)
+                                             (blocked-port pc instance))))
+               (traces (if collateral-blocked? traces
+                           (append-map (cute run-flush <> instance) traces)))
                (traces (map (cute rewrite-trace-head
                                   (cute clone <> #:running-defer? #f) <>)
                             traces))
@@ -820,30 +860,43 @@ until RTC?."
   (extend-trace trace flush-defer))
 
 (define-method (run-external-q (pc <program-counter>) (instance <runtime:port>))
-  (let* ((pc trigger (dequeue-external pc instance))
-         (q-out (make <q-out> #:trigger trigger))
-         (q-out (clone q-out #:location (.location trigger)))
-         (q-out-pc (clone pc #:instance (%sut) #:statement q-out))
-         (traces (run-to-completion pc trigger)))
-    (map (lambda (t) (append t (list q-out-pc))) traces)))
+  (let* ((other-port (runtime:other-port instance))
+         (other-instance (.container other-port)))
+    (define (q-trigger q-trigger)
+      (let* ((port-name (.name (.ast other-port)))
+             (trigger (.trigger pc))
+             (q-trigger (make <q-trigger>
+                          #:event.name (.event.name q-trigger)
+                          #:port.name port-name
+                          #:location (.location q-trigger))))
+        (clone q-trigger #:parent (.type (.ast other-instance)))))
+    (let* ((pc trigger (dequeue-external pc instance))
+           (q-in (make <q-in> #:trigger trigger))
+           (q-in (clone q-in #:location (.location trigger)))
+           (pc (push-pc pc other-instance q-in))
+           (q-trigger (q-trigger trigger))
+           (pc (enqueue pc trigger other-instance q-trigger))
+           (q-in-pc pc)
+           (pc (clone pc #:statement #f))
+           (trace (list pc q-in-pc)))
+      (list trace))))
 
 (define-method (run-external (pc <program-counter>) event)
   (%debug "run-external ~a pc: ~s\n" event pc)
   (let ((queues (.external-q pc)))
-    (if (or (null? queues)
-            (pair? (.async pc))) '()
-            (match (external-trigger-in-q? pc event)
-              ((port q ...)
-               (run-external-q pc (or port (%sut))))
-              (_
-               (let* ((model (runtime:%sut-model))
-                      (ast (if (is-a? model <system>) model
-                               (.behavior model)))
-                      (error (make <match-error> #:ast ast #:input event
-                                   #:message "match"))
-                      (pc (clone pc #:status error)))
-                 (%debug "<match> ~a pc: ~s\n" event pc)
-                 (list (list pc) )))))))
+    (if (null? queues) '()
+        (match (external-trigger-in-q? pc event)
+          ((port q ...)
+           (run-external-q pc (or port (%sut))))
+          (_
+           (let* ((model (runtime:%sut-model))
+                  (ast (if (is-a? model <system>) model
+                           (.behavior model)))
+                  (error (make <match-error> #:ast ast #:input event
+                               #:message "match"))
+                  (pc (clone pc #:status error)))
+             (%debug "<match> ~a pc: ~s\n" event pc)
+             (list (list pc) )))))))
 
 (define-method (run-to-completion* (pc <program-counter>) event)
   (define (illegal-trace pc)
@@ -857,36 +910,34 @@ until RTC?."
    ((is-a? (%sut) <runtime:port>)
     (run-interface pc event))
    ((external-trigger-in-q? pc event)
-    (run-external pc event))
+    (let ((pc (clone pc #:trail (cons event (.trail pc)))))
+      (run-external pc event)))
+   ((and (external-trigger? event)
+         (trigger-in-q? pc event))
+    (run-requires-flush pc event))
    ((external-trigger? event)
-    (let ((pcs (cons pc (run-external-modeling pc event))))
-      (append-map (cute run-external <> event) pcs)))
+    (let* ((pcs (cons pc (run-external-modeling pc event)))
+           (traces (append-map (cute run-external <> event) pcs)))
+      (append-map (cute run-requires-flush <> event) traces)))
    ((requires-trigger? event)
-    (let ((async-traces (if (null? (.async pc)) '()
-                            (flush-async-event pc event))))
-      (if (pair? async-traces) async-traces
-          (let* ((defer? (and (pair? (.defer pc))
-                              (not (%strict?))))
-                 (defer-traces (if (not defer?) '()
-                                   (run-defer-event pc event))))
-            (append
-             defer-traces
-             (let* ((pcs (cons pc (run-external-modeling pc event)))
-                    (port (.port (string->trigger event)))
-                    (blocked-port (and=> (blocked-on-boundary? pc event)
-                                         .ast))
-                    (traces (append-map (cute run-requires <> event) pcs)))
-               (if (or (not port) (not (ast:eq? port blocked-port))) traces
-                   (filter (negate modeling?) traces))))))))
+    (let* ((defer? (and (pair? (.defer pc))
+                        (not (%strict?))))
+           (defer-traces (if (not defer?) '()
+                             (run-defer-event pc event))))
+      (append
+       defer-traces
+       (let* ((pcs (cons pc (run-external-modeling pc event)))
+              (port (.port (string->trigger event)))
+              (blocked-port (and=> (blocked-on-boundary? pc event)
+                                   .ast))
+              (traces (append-map (cute run-requires <> event) pcs)))
+         (if (or (not port) (not (ast:eq? port blocked-port))) traces
+             (filter (negate modeling?) traces))))))
    ((provides-trigger? event)
     (if (blocked-on-boundary-provides? pc event) (illegal-trace pc)
         (run-to-completion pc event)))
-   ((async-event? pc event)
-    (flush-async-event pc event))
    ((defer-event? pc event)
     (run-defer-event pc event))
-   ((and (eq? event #f) (pair? (.async pc)))
-    (flush-async pc))
    (else
     '())))
 
@@ -900,7 +951,8 @@ until RTC?."
                        (.container (runtime:other-port instance))))
          (traces (run-to-completion pc 'rtc))
          (traces (map (cute rewrite-trace-head
-                            (cute clone <> #:instance instance) <>)
+                            (cute clone <> #:instance instance)
+                            <>)
                       traces))
          (done traces
                (partition
@@ -927,8 +979,7 @@ until RTC?."
          (trigger? (or (is-a? (%sut) <runtime:port>)
                        (provides-trigger? event)
                        (and (not blocked-on-action?)
-                            (requires-trigger? event))
-                       (async-event? pc event)))
+                            (requires-trigger? event))))
          (pc (if (or switched? trigger?) pc
                  (blocked-on-boundary-collateral-release pc)))
          (pc (if (provides-trigger? event) pc
