@@ -19,22 +19,19 @@
 ;;; License along with Dezyne.  If not, see <http://www.gnu.org/licenses/>.
 
 (define-module (dzn vm step)
-  #:use-module (ice-9 match)
-
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
   #:use-module (srfi srfi-71)
 
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
-  #:use-module (dzn goops)
+  #:use-module (ice-9 match)
 
+  #:use-module (dzn ast goops)
   #:use-module (dzn ast)
   #:use-module (dzn config)
   #:use-module (dzn misc)
   #:use-module (dzn vm ast)
   #:use-module (dzn vm evaluate)
   #:use-module (dzn vm goops)
-  #:use-module (dzn vm normalize)
   #:use-module (dzn vm runtime)
   #:use-module (dzn vm util)
   #:export (%liveness?
@@ -111,9 +108,10 @@
                              #:message message))))
      ((let* ((r:other-port (runtime:other-port r:port))
              (blocked (append (.blocked pc) (.collateral pc)))
+             (blocked (map (compose rtc-block-pc cdr) blocked))
+             (blocked (filter .instance blocked))
              (ports (map (compose runtime:other-port
-                                  trigger-runtime-port
-                                  rtc-block-pc cdr)
+                                  trigger-runtime-port)
                          blocked)))
         (or (assq r:port (.blocked pc))
             (and (ast:provides? trigger)
@@ -227,6 +225,14 @@
           (else
            (list continuation)))))
 
+(define-method (step (pc <program-counter>) (o <skip>))
+  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
+  (let* ((continuation (continuation pc o))
+         (statement (.statement continuation)))
+    (cond ((mark-liveness? pc statement))
+          (else
+           (list continuation)))))
+
 (define-method (step (pc <program-counter>) (o <if>))
   (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (if (true? (eval-expression pc (.expression o))) (list (continuation pc ((compose list .then) o)))
@@ -259,8 +265,6 @@
   (%debug "  ~s ~s ~a => ~s\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o) (trigger->string o))
   (cond ((ast:out? o)
          (step-action-up pc o))
-        ((ast:async? o)
-         (step-async-action-down pc o))
         (else
          (step-action-down pc o))))
 
@@ -296,34 +300,6 @@
      (else
       (list pc)))))
 
-(define (step-async-action-down pc o)
-  (match (.event.name o)
-    ("req"
-     (let* ((instance (.instance pc))
-            (q-trigger (make <q-trigger> #:port.name (.port.name o) #:event.name "ack"
-                             #:modeling? (ast:modeling? (.trigger pc))))
-            (q-trigger (clone q-trigger
-                              #:parent ((compose .behavior .type .ast) instance)))
-            (ack (lambda (pc) (list (begin-step pc instance q-trigger))))
-            (rank (.rank instance))
-            (r:port (runtime:port instance (.port o)))
-            (req-pending? (find (compose (cute ast:eq? r:port <>) cadr) (.async pc))))
-       (if req-pending?
-           (let ((error (make <illegal-error> #:message "illegal" #:ast o)))
-             (list (clone pc #:status error)))
-           (let ((pc (clone pc #:async (acons rank (cons r:port ack) (.async pc)))))
-             (list (continuation pc o))))))
-    ("clr"
-     (let* ((instance (.instance pc))
-            (r:port (runtime:port instance (.port o)))
-            (timers (.async pc))
-            (canceled (find (compose (cut ast:eq? <> r:port) cadr) (reverse timers)))
-            (timers (reverse (delete canceled timers eq?)))
-            (pc (clone pc #:async timers)))
-       (list (continuation pc o))))
-    (_
-     (throw 'debug "no such async trigger" o))))
-
 (define-method (step-action-up (pc <program-counter>) (o <action>))
   (let* ((instance (.instance pc))
          (port (.port o))
@@ -358,7 +334,7 @@
            (ast:requires? (.ast instance)))
       (list (enqueue pc o other-instance (q-trigger))))
      ((is-a? other-instance <runtime:component>)
-      (list (enqueue pc o  other-instance (q-trigger))))
+      (list (enqueue pc o other-instance (q-trigger))))
      ((runtime:boundary-port? other-port)
       (list pc)))))
 
@@ -385,13 +361,17 @@
                         (.name (.ast instance))))
          (previous (get-reply pc port-name))
          (value (eval-expression pc (.expression o)))
-         (pc (if (not previous) (continuation (set-reply pc port-name value) o)
-                 (let ((error (make <second-reply-error>
-                                #:ast o
-                                #:previous (.parent previous)
-                                #:message "second-reply")))
-                   (%debug "second reply, previous=~a\n" (ast:location->string previous))
-                   (clone pc #:status error))))
+         (pc (cond (previous
+                    (let ((error (make <second-reply-error>
+                                   #:ast o
+                                   #:previous (.parent previous)
+                                   #:message "second-reply")))
+                      (%debug "second reply, previous=~a\n" (ast:location->string previous))
+                      (clone pc #:status error)))
+                   ((is-a? value <void>)
+                    (continuation pc o))
+                   (else
+                    (continuation (set-reply pc port-name value) o))))
          (reply-port (.port o))
          (trigger (.trigger pc)))
     (cond ((let* ((trigger-port (.port trigger))
@@ -401,7 +381,8 @@
                                    stack))
                   (statements (map .statement instance-stack))
                   (blocking? (or (pair? (.blocked pc))
-                                 (any (cute parent <> <blocking>) statements)))
+                                 (any (cute ast:parent <> <blocking>)
+                                      statements)))
                   (ports-eq? (ast:eq? reply-port trigger-port)))
              (and reply-port
                   (or blocking?
@@ -485,6 +466,12 @@
                 (let ((pc (block pc r:port)))
                   (list pc)))))))))
 
+(define-method (step (pc <program-counter>) (o <tag>))
+  (list (continuation pc o)))
+
+(define-method (step (pc <program-counter>) (o <the-end>))
+  (list (continuation pc o)))
+
 (define-method (step (pc <program-counter>) (o <end-of-on>))
   (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (let* ((deferred (pop-deferred pc))
@@ -524,13 +511,12 @@
         (list pc))))))
 
 (define-method (step (pc <program-counter>) (o <trigger-return>))
-  (define (valued-reply? pc o)
+  (define (typed-reply? pc o)
     ;;FIXME: simplify
     (let ((instance (.instance pc))
           (port (.port (.trigger pc))))
       (and (not (and (is-a? instance <runtime:port>)
                      (is-a? (.event (.trigger pc)) <modeling-event>)))
-           (not (and port (ast:async? port)))
            (not (and (is-a? instance <runtime:component>) (not port)))
            (or (is-a? (.instance pc) <runtime:port>)
                (let* ((r:port (runtime:port instance port))
@@ -549,7 +535,7 @@
 
   (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
   (cond
-   ((valued-reply? pc o)
+   ((typed-reply? pc o)
     (let* ((trigger (.trigger pc))
            (type (ast:type trigger))
            (typed? (ast:typed? type))
@@ -560,7 +546,7 @@
    (else
     (let* ((trigger (.trigger pc))
            (blocking? (and (ast:provides? trigger)
-                           (parent o <blocking>)))
+                           (ast:parent o <blocking>)))
            (pc (if (or blocking? (.status pc)) pc
                    (let ((locals (filter (is? <variable>)
                                          (ast:statement* (.parent o)))))
@@ -569,14 +555,6 @@
            (instance (.instance pc))
            (value (and port (get-reply pc (.name port))))
            (pc (cond
-                ((and (not blocking?)
-                      (is-a? (ast:type value) <void>))
-                 (let ((error (make <second-reply-error>
-                                #:ast o
-                                #:previous (.parent value)
-                                #:message "second-reply")))
-                   (%debug "second reply, previous=~a\n" (ast:location->string value))
-                   (clone pc #:status error)))
                 ((not blocking?)
                  pc)
                 (else
@@ -603,19 +581,6 @@
                    pc)))
       (list (pop-pc pc))))))
 
-(define-method (step (pc <program-counter>) (o <flush-async>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
-  (let ((timers (.async pc)))
-    (if (null? timers) (list pc)
-        (let* ((pc (clone pc #:instance (%sut)))
-               (deadline (apply min (map car timers)))
-               (entry (assoc deadline (reverse timers))))
-          (match entry
-            ((timeout port . proc)
-             (let* ((timers (reverse (delete entry (reverse timers) eq?)))
-                    (pc (clone pc #:async timers)))
-               (proc pc))))))))
-
 
 ;;;
 ;;; Continuation
@@ -666,10 +631,10 @@
       (($ <defer>)
        (clone pc #:statement #f))
       (($ <defer-qout>)
-       (let* ((p (parent p <compound>))
+       (let* ((p (ast:parent p <compound>))
               (pc (if (not p) pc
                       (let* ((variables (get-variables pc))
-                             (model (parent o <model>))
+                             (model (ast:parent o <model>))
                              (members (ast:member* model))
                              (locals (drop-right variables (length members))))
                         (pop-locals pc locals)))))
@@ -697,7 +662,7 @@
       (($ <function>)
        (let* ((formals (ast:formal* parent))
               (pc (pop-locals pc formals)))
-        (pop-pc pc)))
+         (pop-pc pc)))
       (_
        (function-return pc parent)))))
 
