@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018 Johri van Eerd <vaneerd.johri@gmail.com>
 ;;; Copyright © 2017 Henk Katerberg <hank@mudball.nl>
 ;;; Copyright © 2021 Rutger van Beusekom <rutger@dezyne.org>
@@ -32,26 +32,34 @@
   #:use-module (guix packages)
   #:use-module (gnu packages base)
   #:use-module (gnu packages bash)
+  #:use-module (gnu packages boost)
   #:use-module (gnu packages guile)
   #:use-module (gnu packages maths)
-  #:use-module (gnu packages pkg-config))
+  #:use-module (gnu packages pkg-config)
+  #:use-module (gnu packages pth)
+  #:use-module (pack scmackerel))
 
 (define-public dezyne
   (package
     (name "dezyne")
-    (version #!dezyne!# "2.17.8")
+    (version #!dezyne!# "2.18.0.rc6")
     (source
      (origin
        (method url-fetch)
-       (uri (string-append "https://dezyne.org/download/dezyne/"
+       (uri (string-append "https://"
+                           (if (string-contains version "rc") "kluit." "")
+                           "dezyne.org/download/dezyne/"
                            name "-" version ".tar.gz"))
        (sha256
-        (base32 #!dezyne!# "0m5cnymcyg7k1jhp4xjpvpay7ia72xdzyjnjg0spsr4p7fy7yql4"))))
+        (base32 #!dezyne!# "0jfcnwwc60j3fgh2h2m7154qcqph8l0zjx86mqzn0lpil1lhqrc1"))))
+    (propagated-inputs (list guile-json-4
+                             guile-readline
+                             scmackerel))
     (inputs (list bash-minimal
+                  boost
                   guile-3.0
-                  guile-json-4
-                  guile-readline
                   mcrl2-minimal
+                  pth
                   sed))
     (native-inputs (list guile-3.0 pkg-config))
     (build-system gnu-build-system)
@@ -77,6 +85,7 @@
                      (json #$(this-package-input "guile-json"))
                      (mcrl2 #$(this-package-input "mcrl2-minimal"))
                      (readline #$(this-package-input "guile-readline"))
+                     (scmackerel #$(this-package-input "scmackerel"))
                      (sed #$(this-package-input "sed"))
                      (effective (read
                                  (open-pipe* OPEN_READ
@@ -90,12 +99,14 @@
                      (scm-path
                       (list (string-append #$output scm-dir)
                             (string-append json scm-dir)
-                            (string-append readline scm-dir)))
+                            (string-append readline scm-dir)
+                            (string-append scmackerel scm-dir)))
                      (go-dir (string-append "/lib/guile/" effective
                                             "/site-ccache/"))
                      (go-path (list (string-append #$output go-dir)
                                     (string-append json go-dir)
-                                    (string-append readline go-dir))))
+                                    (string-append readline go-dir)
+                                    (string-append scmackerel go-dir))))
                 (wrap-program (string-append #$output "/bin/dzn")
                   `("PATH" ":" prefix ,path)
                   `("GUILE_AUTO_COMPILE" ":" = ("0"))
