@@ -1,7 +1,7 @@
 // Dezyne --- Dezyne command line tools
 //
 // Copyright © 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-// Copyright © 2021, 2022 Rutger (regtur) van Beusekom <rutger@dezyne.org>
+// Copyright © 2021, 2022, 2023 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 // Copyright © 2022 Paul Hoogendijk <paul@dezyne.org>
 //
 // This file is part of Dezyne.
@@ -39,18 +39,10 @@ int main ()
   dzn::locator locator;
   dzn::runtime runtime;
   blocking_multiple_provides3 sut (locator.set (runtime));
-  dzn::pump pump;
-  locator.set(pump);
-
-  sut.dzn_meta.name = "sut";
 
   sut.r.in.hello = [&]
   {
-    std::clog << "sut.mp.r.hello -> <external>.r.hello\n";
-    sut.mp.r.out.world (); //side step going through pump, since we are
-                           //already executing on the pump and the
-                           //interface of r is synchronous
-    std::clog << "sut.mp.r.return -> <external>.r.return\n";
+    sut.r.out.world ();
   };
 
   sut.left.in.hello ();
