@@ -1,6 +1,6 @@
 // Dezyne --- Dezyne command line tools
 //
-// Copyright © 2017, 2020, 2022 Rutger van Beusekom <rutger@dezyne.org>
+// Copyright © 2017, 2020, 2022, 2023 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 // Copyright © 2017, 2019, 2020, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 //
 // This file is part of Dezyne.
@@ -35,24 +35,20 @@ main ()
   dzn::locator locator;
   dzn::runtime runtime;
 
-  std::cin.ignore(std::numeric_limits<std::streamsize>::max());
+  std::cin.ignore (std::numeric_limits<std::streamsize>::max ());
 
-  calling_context sut(locator.set(runtime));
-  sut.dzn_meta.name = "sut";
-  sut.h.meta.require.name = "h";
-  sut.w.meta.provide.name = "w";
+  calling_context sut (locator.set (runtime));
 
-  sut.w.in.world = [&](int& cc, int i){
+  sut.w.in.world = [&] (int &cc, int i)
+  {
     (void)i;
-    //std::clog << "cc=" << cc << ", i=" << i << std::endl;
-    dzn::trace (std::clog, sut.w.meta,"world");
-    if(cc == 0){cc = 123;} else {assert(cc == 123); cc = 456;}
-    dzn::trace_out (std::clog, sut.w.meta,"return"); // std::clog << std::endl;
+    if (cc == 0) {cc = 123;}
+    else {assert (cc == 123); cc = 456;}
   };
 
   int cc = 0;
 
-  sut.h.in.hello(cc, 123);
+  sut.h.in.hello (cc, 123);
 
-  assert(cc == 456);
+  assert (cc == 456);
 }
