@@ -1,8 +1,8 @@
 // Dezyne --- Dezyne command line tools
 //
-// Copyright © 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+// Copyright © 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 // Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
-// Copyright © 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
+// Copyright © 2021, 2022, 2023 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 //
 // This file is part of Dezyne.
 //
@@ -31,38 +31,41 @@
 #include <dzn/runtime.hh>
 #include <dzn/pump.hh>
 
-int
-main (int argc, char* argv[])
+static bool
+getopt (int argc, char const *argv[], std::string option)
 {
-  if (argv + argc != std::find_if (argv + 1,
-                                   argv + argc,
-                                   [] (char const* s)
-                                   {return s == std::string ("--debug");}))
+  return argv + argc != std::find_if (argv + 1, argv + argc, [&option] (char const * s)
+  {
+    return s == option;
+  });
+}
+
+int
+main (int argc, char const *argv[])
+{
+  if (getopt (argc, argv, "--debug"))
     dzn::debug.rdbuf (std::clog.rdbuf ());
 
-  dzn::locator loc;
-  dzn::runtime rt;
-  loc.set (rt);
-  collateral_blocking_multiple_provides0 sut (loc);
-  sut.dzn_meta.name = "sut";
-  sut.r.meta.provide.name = "r";
+  dzn::locator locator;
+  dzn::runtime runtime;
+  locator.set (runtime);
+  collateral_blocking_multiple_provides0 sut (locator);
 
   sut.r.in.hello = [&]
   {
-    dzn::trace(std::clog, sut.r.meta, "hello");
-    std::this_thread::sleep_for(std::chrono::milliseconds(200));
+    std::this_thread::sleep_for (std::chrono::milliseconds (200));
     sut.r.out.world();
-    dzn::trace_out(std::clog, sut.r.meta, "return");
   };
 
-  std::thread t([&]{
-    std::this_thread::sleep_for(std::chrono::milliseconds(100));
+  std::thread t([&]
+  {
+    std::this_thread::sleep_for (std::chrono::milliseconds (100));
     sut.right.in.hello ();
   });
 
   sut.left.in.hello ();
 
-  t.join();
+  t.join ();
 
   return 0;
 }
