@@ -2,7 +2,7 @@
 //
 // Copyright © 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 // Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
-// Copyright © 2021 Rutger van Beusekom <rutger@dezyne.org>
+// Copyright © 2021, 2023 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 //
 // This file is part of Dezyne.
 //
@@ -34,34 +34,23 @@
 int
 main ()
 {
+  //dzn::debug.rdbuf (std::clog.rdbuf ());
+
   dzn::locator loc;
   dzn::runtime rt;
   loc.set (rt);
 
-  collateral_blocking_reorder sut(loc);
-  sut.dzn_meta.name = "sut";
-  sut.r.meta.require.name = "r";
-  sut.e.meta.require.name = "e";
+  collateral_blocking_reorder sut (loc);
 
-  bool once = true;
+  bool idle = true;
 
-  sut.r.in.hello = [&]
-  {
-    std::clog << "sut.block.r.hello -> <external>.r.hello" << std::endl;
-    std::thread([&]{
-      if(once) {once = false; sut.e.out.world();}
-      sut.r.out.world();
-    }).detach();
-    std::clog << "sut.block.r.return <- <external>.r.return" << std::endl;
-  };
-  sut.e.in.hello = [&]
-  {
-    std::clog << "sut.proxy.e.hello -> <external>.e.hello" << std::endl;
-    std::clog << "sut.proxy.e.return <- <external>.e.return" << std::endl;
+  sut.e.in.hello = [&] {};
+
+  sut.r.in.hello = [&] {
+    if (idle) sut.e.out.world ();
+    idle = !idle;
+    sut.r.out.world ();
   };
 
   sut.p.in.hello ();
-
-  dzn::pump& pump = sut.dzn_locator.get<dzn::pump>();
-  pump.wait ();
 }
