@@ -2,7 +2,7 @@
 //
 // Copyright © 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 // Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
-// Copyright © 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
+// Copyright © 2021, 2022, 2023 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 //
 // This file is part of Dezyne.
 //
@@ -37,43 +37,31 @@ main ()
 {
   std::cin.ignore (std::numeric_limits<std::streamsize>::max ());
 
-  dzn::locator loc;
-  dzn::runtime rt;
-  loc.set (rt);
-  collateral_blocking_shell sut (loc);
-  sut.dzn_meta.name = "sut";
-  sut.w0.meta.require.name = "w0";
-  sut.w0.meta.require.port = &sut.w0;
-  sut.w1.meta.require.name = "w1";
-  sut.w1.meta.require.port = &sut.w1;
+  dzn::locator locator;
+  dzn::runtime runtime;
+  locator.set (runtime);
+  collateral_blocking_shell sut (locator);
 
   bool cruel = false;
   sut.w0.in.hello = [&]
   {
-    std::clog << "sut.blocked.w0.hello -> <external>.w0.hello\n";
-    std::clog << "<external>.w0.return <- sut.blocked.w0.return\n";
   };
   sut.w1.in.hello = [&]
   {
-    std::clog << "sut.blocked.w1.hello -> <external>.w1.hello\n";
     if (cruel)
       {
         std::thread ([&]
         {
-          std::clog << "cruel\n";
           sut.h.in.cruel ();
-        }).detach();
+        }).detach ();
       }
 
     std::thread ([&]
     {
       std::this_thread::sleep_for (std::chrono::milliseconds (50));
-      std::clog << "world0\n";
       sut.w0.out.world ();
-      std::clog << "world1\n";
       sut.w1.out.world ();
-    }).detach();
-    std::clog << "<external>.w1.return <- sut.blocked.w1.return\n";
+    }).detach ();
   };
 
   std::clog << "hello happy\n";
