@@ -1,6 +1,7 @@
 // Dezyne --- Dezyne command line tools
 //
 // Copyright © 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+// Copyright © 2023 Rutger van Beusekom <rutger@dezyne.org>
 //
 // This file is part of Dezyne.
 //
@@ -23,19 +24,13 @@
 
 #include <foreign_handwritten.hh>
 
-Foreign::Foreign(const dzn::locator& dzn_locator)
-: dzn_meta{"","Foreign",0,{},{},{[this]{w.check_bindings();}}}
-, dzn_rt(dzn_locator.get<dzn::runtime>())
-, dzn_locator(dzn_locator)
-, w({{"w",&w,this,&dzn_meta},{"",0,0,0}})
+Foreign::Foreign (const dzn::locator &dzn_locator)
+  : dzn_meta{"", "Foreign", 0, {}, {}, {[this]{w.dzn_check_bindings ();}}}
+, dzn_runtime (dzn_locator.get<dzn::runtime> ())
+, dzn_locator (dzn_locator)
+, w ({{"w", &w, this, &dzn_meta}, {"", 0, 0, 0}})
 {
   dzn_meta.require = {};
-  w.in.world = [&](){
-    return dzn::call_in(this,[=]{
-      return w_world();
-    },
-    this->w, "world");
-  };
+  w.in.world = [&] {};
+  w.in.world.set (this, &w, "world");
 }
-
-void Foreign::w_world() {}
