@@ -1,7 +1,7 @@
 // Dezyne --- Dezyne command line tools
 //
 // Copyright © 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-// Copyright © 2023 Rutger van Beusekom <rutger@dezyne.org>
+// Copyright © 2023 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 //
 // This file is part of Dezyne.
 //
@@ -30,15 +30,14 @@
 int
 main ()
 {
-  std::cin.ignore(std::numeric_limits<std::streamsize>::max());
+  std::cin.ignore (std::numeric_limits<std::streamsize>::max ());
 
   dzn::locator l;
   dzn::runtime rt;
-  l.set(rt);
+  l.set (rt);
 
-  foreign_optional sut(l);
-  sut.dzn_meta.name = "sut";
+  foreign_optional sut (l);
 
-  sut.c.h.out.world = [] {std::clog << "<external>.h.world <- sut.c.h.world\n";};
-  sut.f.w_hello ();
+  sut.c.h.out.world = [] {};
+  sut.f.w.in.hello ();
 }
