@@ -1,6 +1,7 @@
 // Dezyne --- Dezyne command line tools
 //
 // Copyright © 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+// Copyright © 2023 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 //
 // This file is part of Dezyne.
 //
@@ -28,23 +29,16 @@
 int
 main ()
 {
-  std::cin.ignore(std::numeric_limits<std::streamsize>::max());
+  std::cin.ignore (std::numeric_limits<std::streamsize>::max ());
 
   dzn::locator locator;
   dzn::runtime runtime;
   locator.set (runtime);
 
   foreign_requires sut (locator);
-  sut.dzn_meta.name = "sut";
 
-  sut.c.w0.in.hello = [] {
-    std::clog << "<external>.w0.hello -> sut.c.w0.hello\n";
-    std::clog << "<external>.w0.return <- sut.c.w0.return\n";
-  };
-  sut.c.w1.in.hello = [] {
-    std::clog << "<external>.w1.hello -> sut.c.w1.hello\n";
-    std::clog << "<external>.w1.return <- sut.c.w1.return\n";
-  };
+  sut.w0.in.hello = sut.w0.out.world;
+  sut.w1.in.hello = sut.w1.out.world;
   sut.f.w0_world ();
   sut.f.w1_world ();
 }
