@@ -1,6 +1,6 @@
 // Dezyne --- Dezyne command line tools
 //
-// Copyright © 2022 Rutger van Beusekom <rutger@dezyne.org>
+// Copyright © 2022, 2023 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 //
 // This file is part of Dezyne.
 //
@@ -27,18 +27,17 @@
 #include <dzn/pump.hh>
 #include <dzn/runtime.hh>
 
-int main()
+int main ()
 {
-  for(size_t i = 0; i < 2; ++i)
-  {
-    dzn::locator locator;
-    dzn::runtime runtime;
-    dzn::pump pump;
-    pump_twice sut(locator.set(runtime).set(pump));
-    sut.dzn_meta.name = "sut";
-    sut.h.meta.require.name = "h";
-    sut.h.out.world = [&]{dzn::trace_out(std::clog,sut.h.meta,"world");};
-    dzn::shell(pump, [&]{sut.h.in.hello();});
-    pump.wait();
-  }
+  for (size_t i = 0; i < 2; ++i)
+    {
+      dzn::locator locator;
+      dzn::runtime runtime;
+      dzn::pump pump;
+      pump_twice sut (locator.set (runtime).set (pump));
+
+      sut.h.out.world = [&] {};
+      dzn::shell (pump, [&] {sut.h.in.hello ();});
+      pump.wait ();
+    }
 }
