@@ -1,7 +1,7 @@
 // Dezyne --- Dezyne command line tools
 //
-// Copyright © 2022 Rutger (regtur) van Beusekom <rutger@dezyne.org>
-// Copyright © 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+// Copyright © 2022, 2023 Rutger (regtur) van Beusekom <rutger@dezyne.org>
+// Copyright © 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 //
 // This file is part of Dezyne.
 //
@@ -42,13 +42,19 @@ read ()
   return str;
 }
 
-int
-main (int argc, char **argv)
+static bool
+getopt (int argc, char const *argv[], std::string option)
 {
-  if (argv + argc != std::find_if (argv + 1,
-                                   argv + argc,
-                                   [] (char const* s)
-                                   {return s == std::string ("--debug");}))
+  return argv + argc != std::find_if (argv + 1, argv + argc, [&option] (char const * s)
+  {
+    return s == option;
+  });
+}
+
+int
+main (int argc, char const *argv[])
+{
+  if (getopt (argc, argv, "--debug"))
     dzn::debug.rdbuf (std::clog.rdbuf ());
 
   dzn::locator locator;
@@ -56,48 +62,36 @@ main (int argc, char **argv)
   locator.set (runtime);
   sync_out_reply_binding sut (locator);
 
-  sut.dzn_meta.name = "sut";
-
-  sut.h.meta.require.name = "h";
-  sut.h.meta.require.port = &sut.h;
-
-  sut.w.meta.provide.name = "w";
-  sut.w.meta.provide.port = &sut.w;
-
   sut.w.in.hello = [&]
   {
-    dzn::trace (std::clog, sut.w.meta, "hello");
     sut.w.out.world ();
-    dzn::trace_out (std::clog, sut.w.meta, "return");
   };
 
   sut.w.in.hello_void = [&]
   {
-    dzn::trace (std::clog, sut.w.meta, "hello_void");
     sut.w.out.world_void ();
-    dzn::trace_out (std::clog, sut.w.meta, "return");
   };
 
   std::string trace = read ();
   if (0);
   // trace
   else if (trace == "h.hello\nw.hello\nw.world\nw.return\nh.true")
-  {
-    int v = 0;
-    sut.h.in.hello (v);
-    assert (v == 456);
-  }
+    {
+      int v = 0;
+      sut.h.in.hello (v);
+      assert (v == 456);
+    }
   else if (trace == "h.hello_void\nw.hello_void\nw.world_void\nw.return\nh.return")
-  {
-    int v = 0;
-    sut.h.in.hello_void (v);
-    assert (v == 456);
-  }
+    {
+      int v = 0;
+      sut.h.in.hello_void (v);
+      assert (v == 456);
+    }
   else
-  {
-    std::clog << "missing trace" << std::endl;
-    return 1;
-  }
+    {
+      std::clog << "missing trace" << std::endl;
+      return 1;
+    }
 
   return 0;
 }
