@@ -1,9 +1,10 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2019 Timothy Sample <samplet@ngyro.com>
-;;; Copyright © 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2023 Karol Kobiela <karol.kobiela@verum.com>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -28,11 +29,12 @@
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
   #:use-module (srfi srfi-71)
+
   #:use-module (ice-9 match)
-  #:use-module (ice-9 popen)
   #:use-module (ice-9 rdelim)
   #:use-module (ice-9 receive)
   #:use-module (ice-9 regex)
+
   #:use-module (dzn config)
   #:use-module (dzn misc)
   #:use-module (dzn pipe)
@@ -75,7 +77,9 @@
                                (not (eq? port ready-port)))
                              ps)))
               (chr (accs-cons! chr ready-port)
-                   (loop ps)))))))))
+                   (loop ps))))
+           (('() '() '())
+            (loop ps)))))))
 
 (define (observe command input)
   "Run COMMAND with INPUT, returning the exit status, standard
@@ -147,6 +151,9 @@ output, and standard error as three values."
 ;;;
 ;;; META
 ;;;
+(define (file->model file-name)
+  (string-map (lambda (c)(if (eq? c #\-) #\_ c)) file-name))
+
 (define (get-meta file-name)
   (let ((META (string-append file-name "/META")))
     (and (file-exists? META)
@@ -168,13 +175,29 @@ output, and standard error as three values."
          (and=> (assq-ref alist flag)
                 (cute equal? <> '(#t))))))
 
+(define (fall-back? file-name)
+  (get-meta-flag file-name 'fall-back))
+
+(define (flush? file-name)
+  (get-meta-flag file-name 'flush))
+
+(define (no-unreachable? file-name)
+  (get-meta-flag file-name 'no-unreachable))
+
 (define (queue-size file-name)
   (let ((alist (get-meta file-name)))
     (and alist
          (and=> (assq-ref alist 'queue-size) car))))
 
-(define (flush? file-name)
-  (get-meta-flag file-name 'flush))
+(define (queue-size-defer file-name)
+  (let ((alist (get-meta file-name)))
+    (and alist
+         (and=> (assq-ref alist 'queue-size-defer) car))))
+
+(define (queue-size-external file-name)
+  (let ((alist (get-meta file-name)))
+    (and alist
+         (and=> (assq-ref alist 'queue-size-external) car))))
 
 (define (thread-pool? file-name)
   (get-meta-flag file-name 'thead-pool))
@@ -188,17 +211,23 @@ output, and standard error as three values."
              (and=> (assq-ref alist 'code-options) car))
         '())))
 
+(define (component? file-name)
+  (or (let ((alist (get-meta file-name)))
+        (and alist
+             (and=> (assq-ref alist 'component) car)))
+      (model? file-name)))
+
+(define (component-unset? file-name)
+  (let ((alist (get-meta file-name)))
+    (and alist
+         (and=> (assq 'component alist)
+                (cute equal? <> '(component #f))))))
+
 (define (model? file-name)
   (let ((alist (get-meta file-name)))
     (and alist
          (and=> (assq-ref alist 'model) car))))
 
-(define (model-unset? file-name)
-  (let ((alist (get-meta file-name)))
-    (and alist
-         (and=> (assq 'model alist)
-                (cute equal? <> '(model #f))))))
-
 (define (trace-format file-name)
   (let ((alist (get-meta file-name)))
     (and alist
@@ -226,6 +255,10 @@ output, and standard error as three values."
       (file-exists? (string-append file-name "/baseline/simulate.out"))
       (file-exists? (string-append file-name "/baseline/simulate.err"))))
 
+(define (code-error? file-name)
+  (or (file-exists? (string-append file-name "/baseline/code.out"))
+      (file-exists? (string-append file-name "/baseline/code.err"))))
+
 (define (features file-name)
   (define (feature? feature)
     (and (string-contains file-name feature) feature))
@@ -242,6 +275,10 @@ output, and standard error as three values."
                     (feature? "flush"))
                 "flush")
            (feature? "inject")
+           (feature? "shared")
+           (or (feature? "shell")
+               (and (thread-safe-shell? file-name)
+                    "shell"))
            (feature? "space"))))
 
 (define (features-missing file-name language)
@@ -287,15 +324,31 @@ output, and standard error as three values."
                              '())))
          (includes (filter directory-exists? includes))
          (includes (append-map (cute list "-I" <>) includes))
-         (model (or (model? file-name) base-name))
+         (fall-back? (fall-back? file-name))
+         (model (or (component? file-name) (file->model base-name)))
+         (no-unreachable? (no-unreachable? file-name))
          (queue-size (queue-size file-name))
+         (queue-size-defer (queue-size-defer file-name))
+         (queue-size-external (queue-size-external file-name))
          (command
-          `("dzn" "--verbose" "verify"
-            ,@includes
-            "--all"
-            ,@(if queue-size `("-q" ,(number->string queue-size)) '())
-            ,@(if (model-unset? file-name) '() `("-m" ,model))
-            ,dzn-name)))
+          (if fall-back?
+              `("dzn" "--verbose" "parse" "--fall-back"
+                ,@includes
+                ,dzn-name)
+              `("dzn" "--verbose" "verify"
+                ,@includes
+                "--all"
+                ,@(if (not no-unreachable?) '()
+                      `("--no-unreachable"))
+                ,@(if (not queue-size) '()
+                      `("-q" ,(number->string queue-size)))
+                ,@(if (not queue-size-defer) '()
+                      `("--queue-size-defer" ,(number->string queue-size-defer)))
+                ,@(if (not queue-size-external) '()
+                      `("--queue-size-external"
+                        ,(number->string queue-size-external)))
+                ,@(if (component-unset? file-name) '() `("-m" ,model))
+                ,dzn-name))))
     (or (skip? file-name "verify")
         (run-baseline file-name command
                       #:language "verify"))))
@@ -314,23 +367,25 @@ output, and standard error as three values."
               (includes (append-map (cute list "-I" <>) includes))
               (out (string-append file-name "/out"))
               (out-lang (string-append out "/" language))
-              (model (or (model? file-name) base-name))
-              ;; FIXME: METAs `model' is used for component/system tricksery
-              (model base-name))
+              (model (or (model? file-name) (file->model base-name)))
+              (command
+               `("dzn" "code"
+                 ,@includes
+                 "-l" ,language
+                 "-o" ,out-lang
+                 "-m" ,model
+                 ,@(if (thread-safe-shell? file-name) `("-s" ,base-name) '())
+                 ,@(code-options file-name)
+                 ,dzn-name)))
          (or (error-model? file-name)
              (skip? file-name "code" language (string-append language ":code"))
              (feature-skip? file-name language)
+             (and (code-error? file-name)
+                  (run-baseline file-name command
+                                #:language "code"))
              (and
               (receive (status stdout stderr)
-                  (observe
-                   `("dzn" "code"
-                     ,@includes
-                     "-l" ,language
-                     "-o" ,out-lang
-                     "-m" ,model
-                     ,@(if (thread-safe-shell? file-name) `("-s" ,base-name) '())
-                     ,@(code-options file-name)
-                     ,dzn-name) input)
+                  (observe command input)
                 (zero? status))
               (let* ((dzn-files (append (list-files file-name ".+\\.*dzn*$")
                                         (if (not (directory-exists? in-lang)) '()
@@ -347,9 +402,11 @@ output, and standard error as three values."
                           ,dzn-file) input)
                      (zero? status)))
                  extra-dzn-files)))))
-       (run-build file-name language)
-       (let ((traces (find-files file-name ".*trace.*$")))
-         (and-map (cute run-execute file-name language <>) traces))))
+       (or (code-error? file-name)
+           (run-build file-name language))
+       (or (code-error? file-name)
+           (let ((traces (find-files file-name ".*trace.*$")))
+             (and-map (cute run-execute file-name language <>) traces)))))
 
 (define (run-traces file-name)
   (format #t "** stage: traces\n")
@@ -362,8 +419,10 @@ output, and standard error as three values."
                              '())))
          (includes (filter directory-exists? includes))
          (includes (append-map (cute list "-I" <>) includes))
-         (model (or (model? file-name) base-name))
+         (model (or (component? file-name) (file->model base-name)))
          (queue-size (queue-size file-name))
+         (queue-size-defer (queue-size-defer file-name))
+         (queue-size-external (queue-size-external file-name))
          (language "traces")
          (out (string-append file-name "/out"))
          (out-lang (string-append out "/" language))
@@ -379,7 +438,13 @@ output, and standard error as three values."
                "-o" ,out-lang
                ,@(if (null? handwritten-traces) '("--traces") '())
                ,@(if (flush? file-name) '("--flush") '())
-               ,@(if queue-size `("-q" ,(number->string queue-size)) '())
+               ,@(if (not queue-size) '()
+                     `("-q" ,(number->string queue-size)))
+               ,@(if (not queue-size-defer) '()
+                     `("--queue-size-defer" ,(number->string queue-size-defer)))
+               ,@(if (not queue-size-external) '()
+                     `("--queue-size-external" ,(number->string
+                                                 queue-size-external)))
                "--lts"
                ,dzn-name)
              input)
@@ -398,7 +463,7 @@ are weak-bisim equivalent"
                              '())))
          (includes (filter directory-exists? includes))
          (includes (append-map (cute list "-I" <>) includes))
-         (model (or (model? file-name) base-name))
+         (model (or (component? file-name) (file->model base-name)))
          (queue-size (queue-size file-name))
          (language "constrained-component")
          (out (string-append file-name "/out"))
@@ -444,15 +509,20 @@ are weak-bisim equivalent"
                   ,@(if queue-size `("-q" ,(number->string queue-size)) '())
                   ,dzn-name)
                 input)))
-          (with-output-to-file constrained (cute display stdout-constrained))
-          (with-output-to-file unconstrained (cute display stdout-unconstrained))
+          (with-output-to-file constrained
+            (cute display stdout-constrained))
+          (with-output-to-file unconstrained
+            (cute display stdout-unconstrained))
           (and (zero? status-constrained)
                (zero? status-unconstrained)
                (let ((status
                       stdout
                       stderr
                       (observe
-                       `("ltscompare" "--structured-output" "-eweak-bisim" "--tau=constrained_legal"
+                       `("ltscompare"
+                         "--structured-output"
+                         "-eweak-bisim"
+                         "--tau=constrained_legal"
                          ,constrained
                          ,unconstrained)
                        input)))
@@ -471,7 +541,7 @@ are weak-bisim equivalent"
                              '())))
          (includes (filter directory-exists? includes))
          (includes (append-map (cute list "-I" <>) includes))
-         (model (or (model? file-name) base-name))
+         (model (or (component? file-name) (file->model base-name)))
          (queue-size (queue-size file-name))
          (language "constrained-no-compliance")
          (out (string-append file-name "/out"))
@@ -517,15 +587,20 @@ are weak-bisim equivalent"
                   ,@(if queue-size `("-q" ,(number->string queue-size)) '())
                   ,dzn-name)
                 input)))
-          (with-output-to-file constrained (cute display stdout-constrained))
-          (with-output-to-file no-compliance (cute display stdout-no-compliance))
+          (with-output-to-file constrained
+            (cute display stdout-constrained))
+          (with-output-to-file no-compliance
+            (cute display stdout-no-compliance))
           (and (zero? status-constrained)
                (zero? status-no-compliance)
                (let ((status
                       stdout
                       stderr
                       (observe
-                       `("ltscompare" "--structured-output" "-eweak-bisim" "--tau=constrained_legal"
+                       `("ltscompare"
+                         "--structured-output"
+                         "-eweak-bisim"
+                         "--tau=constrained_legal"
                          ,constrained
                          ,no-compliance)
                        input)))
@@ -575,11 +650,16 @@ are weak-bisim equivalent"
             (observe `(,@interpreter
                        ,test ,@(if (flush? file-name) '("--flush") '())) input)
           (and (zero? status)
-               (let ((net (trace:format-trace stderr #:format "event")))
+               (let* ((meta? (member language '("c++")))
+                      (net (trace:format-trace stderr #:format "event" #:meta? meta?))
+                      (net (filter-<defer> net))
+                      (net (if meta? net
+                               (filter-<flush> net))))
                  (receive (status stdout stderr)
                      (let* ((input (filter-state input))
-                            (input (filter-<flush> input))
-                            (input (filter-<defer> input)))
+                            (input (filter-<defer> input))
+                            (input (if meta? input
+                                       (filter-<flush> input))))
                        (observe `("bash" "-c"
                                   ,(string-append "diff -ywB"
                                                   ;; ignore foreign/system communications
@@ -601,15 +681,14 @@ are weak-bisim equivalent"
                              '())))
          (includes (filter directory-exists? includes))
          (includes (append-map (cute list "-I" <>) includes))
-         (model (or (model? file-name) base-name))
+         (model (or (model? file-name) (file->model base-name)))
          (language "simulate")
          (out (string-append file-name "/out"))
          (out-lang (string-append out "/" language))
          (input (filter-<flush> input))
-         (model (or (model? file-name) base-name))
          (queue-size (queue-size file-name))
-         ;; FIXME: METAs `model' is used for component/system tricksery
-         (model base-name)
+         (queue-size-defer (queue-size-defer file-name))
+         (queue-size-external (queue-size-external file-name))
          (baseline? (file-exists? (string-append baseline ".out")))
          (trace-format (cond ((trace-format file-name))
                              (baseline? "trace")
@@ -617,7 +696,14 @@ are weak-bisim equivalent"
     (receive (status stdout stderr)
         (observe `("dzn" "simulate"
                    ,@includes
-                   ,@(if queue-size `("-q" ,(number->string queue-size)) '())
+                   ,@(if (not queue-size) '()
+                         `("-q" ,(number->string queue-size)))
+                   ,@(if (not queue-size-defer) '()
+                         `("--queue-size-defer"
+                           ,(number->string queue-size-defer)))
+                   ,@(if (not queue-size-external) '()
+                         `("--queue-size-external"
+                           ,(number->string queue-size-external)))
                    "--format" ,trace-format
                    ,@(if (non-strict? file-name) '() '("--strict"))
                    ,@(simulate-flags file-name)
@@ -672,8 +758,10 @@ are weak-bisim equivalent"
                                     (cute string-split <> #\:))
                              '())))
          (includes (append-map (cute list "-I" <>) includes))
-         (model (or (model? file-name) base-name))
+         (model (or (component? file-name) (file->model base-name)))
          (queue-size (queue-size file-name))
+         (queue-size-defer (queue-size-defer file-name))
+         (queue-size-external (queue-size-external file-name))
          (language "lts")
          (out (string-append file-name "/out"))
          (out-lang (string-append out "/" language))
@@ -687,7 +775,13 @@ are weak-bisim equivalent"
             (observe
              `("dzn" "graph" "--backend=lts"
                ,@includes
-               ,@(if queue-size `("-q" ,(number->string queue-size)) '())
+               ,@(if (not queue-size) '()
+                     `("-q" ,(number->string queue-size)))
+               ,@(if (not queue-size-defer) '()
+                     `("--queue-size-defer" ,(number->string queue-size-defer)))
+               ,@(if (not queue-size-external) '()
+                     `("--queue-size-external"
+                       ,(number->string queue-size-external)))
                "-m" ,model
                ,dzn-name)
              input)
